
import wx.adv
import wx
from .wxControl import *

class 动画框(wx.adv.AnimationCtrl, 公用方法):
    pass

    def 创建控件动画对象(self):
        return self.CreateCompatibleAnimation()

    def 创建控件动画对象2(self):
        return self.CreateAnimation()

    def 取当前动画(self):
        return self.GetAnimation()

    def 取当前图片(self):
        '返回当此控件显示的非活动位图；查看SetInactiveBitmap 更多信息'
        return self.GetInactiveBitmap()

    def 是否正在播放动画(self):
        return self.IsPlaying()

    @组件_异常检测
    def 载入动画_流(self,文件):
        '从给定的流中加载动画并调用SetAnimation'
        return self.Load(文件)

    @组件_异常检测
    def 载入动画_文件(self,文件):
        '从给定的文件加载动画并调用SetAnimation。'
        return self.LoadFile(文件)

    def 播放动画(self):
        return self.Play()

    def 停止播放(self):
        return self.Stop()

    @组件_异常检测
    def 载入动画(self,动画):
        '设置动画在此控件中播放'
        return self.SetAnimation(动画)

    @组件_异常检测
    def 置默认显示图片(self,图片):
        '设置位图在不播放动画时显示在控件上。'
        return self.SetInactiveBitmap(图片)



"""

.. Hint::
    系统处理


.. literalinclude:: ../../../pyefun/systemProcessingBase_test.py
    :language: python
    :caption: 代码示例
    :linenos:

"""

import time
import os
import platform



def 系统_是否为window系统():
    return platform.system().lower() == 'windows'


def 系统_是否为linux系统():
    return platform.system().lower() == 'linux'

def 系统_是否为mac系统():
    return platform.system().lower() == 'darwin'


# 运行
# 打开内存文件
# 取剪辑板文本
# 置剪辑板文本
# 剪辑板中可有文本
# 清除剪辑板
# 取屏幕宽度
# 取屏幕高度
# 取鼠标水平位置
# 取鼠标垂直位置
# 取颜色数
# 输入框
# 信息框
# 取文本注册项
# 取数值注册项
# 取字节集注册
# 写注册项
# 删除注册项
# 注册项是否存在
# 取默认底色
# 快照
# 读配置项
# 写配置项
# 取配置节名
# 取操作系统类别
# 多文件对话框


def 延时(秒: int):
    time.sleep(秒)


def 运行(cmd):
    p = os.popen(cmd)
    x = p.read()
    p.close()
    return x






"""

.. Hint::
    图像处理

    pip install pillow

.. literalinclude:: ../../../pyefun/imageUtil_test.py
    :language: python
    :caption: 代码示例
    :linenos:


"""
from PIL import Image
import io


def image到字节集(image对象):
    字节流 = io.BytesIO()
    image对象.save(字节流, format='PNG')
    图片字节集 = 字节流.getvalue()
    字节流.close()
    return 图片字节集


def image从字节集加载(图片字节集) -> Image:
    return Image.open(io.BytesIO(图片字节集))


def image取图片宽度高度(image对象: Image.Image):
    width, height = image对象.size
    return width, height

def image显示图片(image对象):
    image对象.show()


"""

.. Hint::
    缓存工具 用于快速缓存函数结果的

.. literalinclude:: ../../../pyefun/cacheUtil_test.py
    :language: python
    :caption: 代码示例
    :linenos:
    :lines: 1-100

"""


import ubelt as ub
from os.path import basename


class 缓存(ub.Cacher):
    def __init__(self, 前缀='cache',
                 key="",
                 缓存目录="./cache/",
                 扩展名=".pkl",
                 启用=True,
                 信息级别=3,
                 哈希类型="sha1",
                 协议版本=4,
                 ):
        "显示 0 不显示 1结束显示 2概率显示 3全部显示"
        # ProgIter(iterable=迭代对象,total=总数, desc=描述, show_times=False, verbose=显示)
        super().__init__(
            fname=前缀,
            depends=ub.hash_data(key),
            dpath=缓存目录,
            # appname="my",
            ext=扩展名,
            verbose=信息级别,
            enabled=启用,
            hasher=哈希类型,
            protocol=协议版本,
        )

    def 取缓存文件路径(self, cfgstr=None):
        return self.get_fpath(cfgstr)

    def 是否存在(self, cfgstr=None):
        return self.exists(cfgstr)

    def 取缓存文件列表(self, cfgstr=None):
        exist_fnames = list(map(basename, self.existing_versions()))
        return exist_fnames

    def 清空(self, cfgstr=None):
        return self.clear(cfgstr)

    def 读入(self, cfgstr=None):
        return self.tryload(cfgstr)

    def 保存(self, data, cfgstr=None):
        self.save(data, cfgstr)


class 缓存标记(ub.CacheStamp):
    def __init__(self, 前缀='cache',
                 key="",
                 缓存目录="./cache/",
                 启用=True,
                 信息级别=3,
                 哈希类型="sha1",
                 ):
        "显示 0 不显示 1结束显示 2概率显示 3全部显示"
        # ProgIter(iterable=迭代对象,total=总数, desc=描述, show_times=False, verbose=显示)
        super().__init__(
            fname=前缀,
            depends=ub.hash_data(key),
            dpath=缓存目录,
            # appname="my",
            verbose=信息级别,
            enabled=启用,
            hasher=哈希类型,
        )

    def 检查标记(self, cfgstr=None):
        return self.expired(cfgstr)

    def 保存标记(self, cfgstr=None, product=None):
        return self.renew(cfgstr, product)



"""
*******************
浏览器类
*******************

.. contents:: 目录


.. Hint::
    需要安装 selenium chrome浏览器或者Firefox浏览器

    本地调试下需要安装 chrome浏览器 chromedriver驱动 或者 Firefox geckodriver驱动

    window macos ubuntu  使用 浏览器初始化本地环境() 就可以自动检测驱动和自动下载配置

    使用 docker 部署远程浏览器 无需安装驱动

    部署时建议使用docker部署远程浏览器

安装 selenium
================

``pip install selenium``


chromedriver 驱动下载
================

https://chromedriver.chromium.org/

https://chromedriver.storage.googleapis.com/index.html?path=90.0.4430.24/

Firefox 驱动下载
================

https://github.com/mozilla/geckodriver/releases

docker部署远程浏览器
================

linux
-------------------

    ``docker run --name chrome -d -p 4444:4444 -p 5900:5900 -v /dev/shm:/dev/shm selenium/standalone-chrome-debug:3.141.59-20210422``

window
-------------------

    ``docker run --name chrome -d -p 4444:4444 -p 5900:5900 selenium/standalone-chrome-debug:3.141.59-20210422``

vpc连接远程桌面
-------------------

VNC Viewer
https://www.realvnc.com/en/connect/download/viewer/

127.0.0.1:5900 即可连接至远程桌面


.. image:: /_static/vnc_viewer.png



.. literalinclude:: ../../../pyefun/seleniumUtil/seleniumUtil_test.py
    :language: python
    :caption: 代码示例
    :linenos:


"""

from selenium import webdriver
from selenium.webdriver.common.desired_capabilities import DesiredCapabilities
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.support.select import Select

from selenium.common.exceptions import TimeoutException
from selenium.webdriver.support.ui import WebDriverWait

from selenium.webdriver.common.by import By  # 等待页面加载延时模块
from selenium.webdriver.support import expected_conditions as EC  # 等待页面加载延时模块

from pyefun import *
import requests

键盘_F1 = '\ue031'
键盘_F2 = '\ue032'
键盘_F3 = '\ue033'
键盘_F4 = '\ue034'
键盘_F5 = '\ue035'
键盘_F6 = '\ue036'
键盘_F7 = '\ue037'
键盘_F8 = '\ue038'
键盘_F9 = '\ue039'
键盘_F10 = '\ue03a'
键盘_F11 = '\ue03b'
键盘_F12 = '\ue03c'

键盘_META = '\ue03d'
键盘_COMMAND = '\ue03d'
键盘_ESCAPE = '\ue00c'
键盘_SPACE = '\ue00d'
键盘_PAUSE = '\ue00b'

键盘_向左 = '\ue012'
键盘_向上 = '\ue013'
键盘_向右 = '\ue014'
键盘_向下 = '\ue015'

键盘_INSERT = '\ue016'
键盘_DELETE = '\ue017'
键盘_PAGE_UP = '\ue00e'
键盘_PAGE_DOWN = '\ue00f'
键盘_END = '\ue010'
键盘_HOME = '\ue011'
键盘_ALT = '\ue00a'
键盘_CTRL = '\ue009'
键盘_退格 = '\ue003'
键盘_空格 = '\ue00d'
键盘_TAB = '\ue004'
键盘_ESC = '\ue00c'
键盘_回车 = '\ue007'
键盘_SHIFT = '\ue008'


def 浏览器初始化本地环境():
    """
.. Hint::
    自动检测环境 自动下载驱动 如果使用远程浏览器则无需安装驱动 也无需调用本函数

    """
    if 系统_是否为window系统():
        返回内容 = 运行("chromedriver -v")
        if 判断文本(返回内容, ["ChromeDriver"]) == False:
            驱动路径 = "C:/Windows/chromedriver.exe"
            if 文件是否存在(驱动路径) == False:
                保存路径 = 取运行目录() + "/chromedriver.zip"
                下载地址 = "https://chromedriver.storage.googleapis.com/90.0.4430.24/chromedriver_win32.zip"
                print("chromedriver 驱动不存在 正在自动下载 %s 保存路径为:%s" % (下载地址, 保存路径))
                下载文件(下载地址, 保存路径)
                解压目录 = "C:/Windows"
                zip解压(保存路径, 解压目录)
                print("chromedriver 解压至:%s" % (驱动路径))
                文件_删除(保存路径)

    if 系统_是否为mac系统():
        返回内容 = 运行("chromedriver -v")
        if 判断文本(返回内容, ["ChromeDriver"]) == False:
            驱动路径 = "/usr/local/bin/chromedriver"
            if 文件是否存在(驱动路径) == False:
                保存路径 = 取运行目录() + "/chromedriver.zip"
                下载地址 = "https://chromedriver.storage.googleapis.com/90.0.4430.24/chromedriver_mac64.zip"
                print("chromedriver 驱动不存在 正在自动下载 %s 保存路径为:%s" % (下载地址, 保存路径))
                下载文件(下载地址, 保存路径)
                解压目录 = "/usr/local/bin/"
                zip解压(保存路径, 解压目录)
                print("chromedriver 解压至:%s" % (驱动路径))
                文件_删除(保存路径)
                文件_修改权限(驱动路径, stat.S_IRWXU)
    if 系统_是否为linux系统():
        返回内容 = 运行("chromedriver -v")
        if 判断文本(返回内容, ["ChromeDriver"]) == False:
            驱动路径 = "/usr/local/bin/chromedriver"
            if 文件是否存在(驱动路径) == False:
                保存路径 = 取运行目录() + "/chromedriver.zip"
                下载地址 = "https://chromedriver.storage.googleapis.com/90.0.4430.24/chromedriver_linux64.zip"
                print("chromedriver 驱动不存在 正在自动下载 %s 保存路径为:%s" % (下载地址, 保存路径))
                下载文件(下载地址, 保存路径)
                解压目录 = "/usr/local/bin/"
                zip解压(保存路径, 解压目录)
                print("chromedriver 解压至:%s" % (驱动路径))
                文件_删除(保存路径)
                文件_修改权限(驱动路径, stat.S_IRWXU)


class 浏览器类():

    def 远程浏览器是否就绪(self, 远程浏览器地址="http://127.0.0.1:4444/wd/hub"):
        """
            while 浏览器.远程浏览器是否就绪(远程浏览器地址) == False:
                延时(1)
                print("浏览器未就绪")
        """
        try:
            data = requests.get(远程浏览器地址 + "/status")
            json = data.json()
            ready = json['value']['ready']
        except:
            return False
        return ready

    def 获取远程chrome(self, server_url="http://127.0.0.1:4444/wd/hub"):
        capabilities = DesiredCapabilities.CHROME.copy()
        capabilities['goog:chromeOptions'] = {"args": ["--headless"]}
        driver = webdriver.Remote(
            command_executor=server_url,
            desired_capabilities=capabilities
        )
        self.浏览器 = driver
        return driver

    def 获取本地chrome(self, 驱动路径='chromedriver', ChromeOptions=webdriver.ChromeOptions()):
        # opt = webdriver.ChromeOptions()
        # # opt.add_argument('--headless')
        # opt.add_argument('--disable-gpu')
        # opt.add_argument("blink-settings=imagesEnabled=false")
        # opt.add_argument(
        #     "user-agent='Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/62.0.3202.94 Safari/537.36'")
        driver = webdriver.Chrome(驱动路径, options=ChromeOptions)
        self.浏览器 = driver
        return driver

    def 获取本地Firefox(self):
        opt = webdriver.FirefoxOptions()
        opt.add_argument('--headless')
        opt.add_argument('--disable-gpu')
        opt.add_argument("blink-settings=imagesEnabled=false")
        opt.add_argument(
            "user-agent='Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/90.0.4430.212 Safari/537.36'")
        driver = webdriver.Firefox(options=opt)
        self.浏览器 = driver
        return driver

    def 打开chrome(self, 驱动路径='chromedriver', 无痕模式=True, 无头模式=False, 隐藏自动化提示=True, 隐藏滚动条=False, 禁止加载图片=False,
                 禁用插件=False,
                 禁止Javascript=False, 禁用JAVA=False, 禁止密码保存提示=True, 禁用弹窗=False, 禁止策略化=False, 初始运行=False, 沙盒运行=True,
                 user_agent='Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/90.0.4430.212 Safari/537.36',
                 代理='', 编码='lang=zh_CN.UTF-8', 禁止使用GPU=False, 谷歌浏览器路径=''):
        '代理格式：127.0.0.1:8888  带账号密码验证的自行百度修改'

        chrome_options = webdriver.ChromeOptions()
        if 无痕模式:
            chrome_options.add_argument('--incognito')
        if 无头模式:
            chrome_options.add_argument('--headless')
        if 隐藏自动化提示:
            chrome_options.add_experimental_option("excludeSwitches", ['enable-automation'])
        if 隐藏滚动条:
            chrome_options.add_argument('--hide-scrollbars')
        if 禁止加载图片:
            chrome_options.add_argument('blink-settings=imagesEnabled=false')
        if 禁用插件:
            chrome_options.add_argument('--disable-plugins')
        if 禁止Javascript:
            chrome_options.add_argument('--disable-javascript')
        if 禁止密码保存提示:
            prefs = {}
            prefs["credentials_enable_service"] = False
            prefs["profile.password_manager_enabled"] = False
            chrome_options.add_experimental_option("prefs", prefs)
        if 禁止策略化:
            chrome_options.add_argument('--disable-infobars')
        if 禁用弹窗:
            chrome_options.add_argument('--disable-popup-blocking')
        if 禁用JAVA:
            chrome_options.add_argument('–disable-java')
        if not 沙盒运行:
            chrome_options.add_argument('--no-sandbox')  # 解决DevToolsActivePort文件不存在的报错
        if user_agent:
            chrome_options.add_argument('User-Agent=' + user_agent)
        if 编码:
            chrome_options.add_argument(编码)
        if 禁止使用GPU:
            chrome_options.add_argument('--disable-gpu')  # 谷歌文档提到需要加上这个属性来规避bug
        if 代理:
            chrome_options.add_argument('--proxy-server=http://' + 代理)
        if 谷歌浏览器路径:
            chrome_options.binary_location = 谷歌浏览器路径
        if 初始运行:
            chrome_options.add_argument('–first run')

        self.获取本地chrome(驱动路径, chrome_options)

        return True

    def 浏览网页(self, URL地址, 隐藏Navigator=True):
        """
        浏览指定网页。本命令为初级对象成员命令。
        """
        self.浏览器.get(URL地址)
        if 隐藏Navigator:
            script = "Object.defineProperty(navigator,'webdriver',{get: ()=> false,});"
            self.浏览器.execute_script(script)
        return True

    def 等待元素加载(self, By元素类型=0, 元素名称="", 等待秒数=60):
        """成功返回真,失败返回假 ,By元素类型参数0是css_selector 谷歌直接复制 1 是id属性 2是name属性, 3 是链接名称 4是xpath属性 5是class_name属性 6是tag_name属性 7,模糊 链接名称"""
        if By元素类型 == 0:
            a = By.CSS_SELECTOR
        elif By元素类型 == 1:
            a = By.ID
        elif By元素类型 == 2:
            a = By.NAME
        elif By元素类型 == 3:
            a = By.LINK_TEXT
        elif By元素类型 == 4:
            a = By.XPATH
        elif By元素类型 == 5:
            a = By.CLASS_NAME
        elif By元素类型 == 6:
            a = By.TAG_NAME
        elif By元素类型 == 7:
            a = By.PARTIAL_LINK_TEXT
        try:
            Wait(self.浏览器, 等待秒数).until(EC.presence_of_element_located((a, 元素名称)))  # 等待页面id为 password 是否加载 完成
            return True
        except Exception as cuowu:
            return False

    def 等待元素显示(self, 等待时间=5, className=""):
        try:
            WebDriverWait(self.浏览器, 等待时间, 0.5, ignored_exceptions=TimeoutException).until(
                lambda x: x.find_element_by_class_name(className).is_displayed())
        except:
            return False
        return True

    def 取页面标题(self):
        return self.浏览器.title

    def 取页面链接(self):
        return self.浏览器.current_url

    def 取页面源码(self):
        return self.浏览器.page_source

    def 关闭当前页面(self):
        '如果只有一个页面标签则会关闭浏览器'
        self.浏览器.close()
        return True

    def 退出(self):
        """
        关闭浏览器
        """
        self.浏览器.quit()
        return True

    def 全屏(self):
        '等于按了F11 浏览器窗口全屏显示'
        self.浏览器.fullscreen_window()
        return True

    def 置浏览器宽度高度(self, 宽度, 高度):
        self.浏览器.set_window_size(宽度, 高度)
        return True

    def 后退(self):
        '控制浏览器后退'
        self.浏览器.back()
        return True

    def 前进(self):
        '控制浏览器前进'
        self.浏览器.forward()
        return True

    def 刷新页面(self):
        self.浏览器.refresh()
        return True

    def 进入Frame页面(self, Frame标识):
        "默认可以直接取表单的id 或name属性。如果iframe没有可用的id和name属性可以通过定位获取的标识直接传入"
        self.浏览器.switch_to.frame(Frame标识)
        return True

    def 置最外层Frame框架(self):

        self.浏览器.switch_to.frame_content()

    def 置上一层Frame框架(self):

        self.浏览器.switch_to.partent_frame()

    def 退出Frame页面(self):
        self.浏览器.switch_to.default_content()
        return True

    def 取所有页面句柄(self):
        '返回所有窗口的句柄到当前会话'
        return self.浏览器.window_handles

    def 取当前页面句柄(self):
        '获得当前窗口句柄'
        return self.浏览器.current_window_handle

    def 切换页面(self, 页面句柄):
        '用于不同窗口的切换'
        self.switch_to.window(页面句柄)
        return True

    def 提示框同意(self):
        '接受现有警告框'
        self.浏览器.switch_to.alert.accept()
        return True

    def 提示框取消(self):
        '解散现有警告框'
        self.浏览器.switch_to.alert.dismiss()
        return True

    def 提示框发送文本(self, 内容):
        '发送文本至警告框。keysToSend：将文本发送至警告框'
        self.浏览器.switch_to.alert.send_keys(内容)
        return True

    def 提示框取文本(self):
        '返回 alert/confirm/prompt 中的文字信息'
        return self.浏览器.switch_to.alert.text

    def 提示框置焦点(self):
        '返回 alert/confirm/prompt 中的文字信息'
        return self.浏览器.switch_to.alert()

    def 取元素从id(self, id):
        '通过元素id定位,匹配多个则返回对象列表'
        结果 = self.浏览器.find_elements_by_id(id)
        if len(结果) == 0:
            return False
        return 浏览器元素操作(self.浏览器, 结果[0])

    def 取元素从name(self, name):
        '通过元素name定位,匹配多个则返回对象列表'
        return [浏览器元素操作(self.浏览器, x) for x in self.浏览器.find_elements_by_name(name)]

    def 取元素从xpath(self, xpath):
        '通过xpath表达式定位,匹配多个则返回对象列表'
        return [浏览器元素操作(self.浏览器, x) for x in self.浏览器.find_elements_by_xpath(xpath)]

    def 取元素从link_text(self, text):
        '通过完整超链接定位,匹配多个则返回对象列表'
        return [浏览器元素操作(self.浏览器, x) for x in self.浏览器.find_elements_by_link_text(text)]

    def 取元素从partial_link_text(self, text):
        '通过部分链接定位,匹配多个则返回对象列表'
        return [浏览器元素操作(self.浏览器, x) for x in self.浏览器.find_elements_by_partial_link_text(text)]

    def 取元素从tag_name(self, tagname):
        '通过标签定位,匹配多个则返回对象列表'
        return [浏览器元素操作(self.浏览器, x) for x in self.浏览器.find_elements_by_tag_name(tagname)]

    def 取元素从class_name(self, name):
        '通过类名进行定位,匹配多个则返回对象列表'
        return [浏览器元素操作(self.浏览器, x) for x in self.浏览器.find_elements_by_class_name(name)]

    def 取元素从css_selector(self, css):
        '通过css选择器进行定位,匹配多个则返回对象列表'
        return [浏览器元素操作(self.浏览器, x) for x in self.浏览器.find_elements_by_css_selector(css)]

    def 最大化(self):
        "浏览器窗口最大化"
        self.浏览器.maximize_window()
        return True

    def 最小化(self):
        "浏览器窗口最小化"
        self.浏览器.minimize_window()
        return True

    def 取名称(self):
        "查看浏览器的名字"
        return self.浏览器.name

    def 取窗口坐标(self):
        "获取当前窗口的坐标"
        return self.浏览器.get_window_position()

    def 取窗口矩形(self):
        "获取窗口的x，y坐标以及当前窗口的高度和宽度"
        return self.浏览器.get_window_rect()

    def 取窗口宽度高度(self):
        "获取当前窗口的尺寸"
        return self.浏览器.get_window_size()

    def 取所有Cookie(self):
        '获得所有cookie信息'
        return self.浏览器.get_cookies()

    def 取指定Cookie(self, name):
        '返回字典的key为“name”的cookie信息'
        return self.浏览器.get_cookie(name)

    def 添加Cookie(self, cookie_dict):
        '添加cookie。“cookie_dict”指字典对象，必须有name 和value 值'
        self.浏览器.add_cookie(cookie_dict)
        return True

    def 删除指定Cookie(self, name):
        '删除cookie信息。“name”是要删除的cookie的名称'
        self.浏览器.delete_cookie(name)
        return True

    def 删除所有Cookie(self):
        '删除所有cookie信息'
        self.浏览器.delete_all_cookies()
        return True

    def 运行JS(self, *args):
        '执行js代码,算加密需要自己声明调用的对象'
        return self.浏览器.execute_script(*args)

    def 运行JS异步(self, *args):
        '执行js代码'
        return self.浏览器.execute_async_script(*args)

    def 新建页面(self, URL):
        '通过javascript打开新页面'
        js = "window.open('{}')".format(URL)
        self.浏览器.execute_script(js)
        return True

    def 置滚动条位置(self, 横向=0, 纵向=0):
        '通过javascript设置浏览器窗口的滚动条位置'
        js = "window.scrollTo({},{});".format(横向, 纵向)
        self.浏览器.execute_script(js)
        return True

    def 滚动条靠左(self):
        '通过javascript设置浏览器窗口的滚动条位置'
        js = "var q=document.documentElement.scrollLeft=0"
        self.浏览器.execute_script(js)
        return True

    def 滚动条靠右(self):
        '通过javascript设置浏览器窗口的滚动条位置'
        js = "var q=document.documentElement.scrollLeft=10000"
        self.浏览器.execute_script(js)
        return True

    def 滚动条靠顶(self):
        '通过javascript设置浏览器窗口的滚动条位置'
        js = "var q=document.documentElement.scrollTop=0"
        self.浏览器.execute_script(js)
        return True

    def 滚动条靠底(self):
        '通过javascript设置浏览器窗口的滚动条位置'
        js = "var q=document.documentElement.scrollTop=10000"
        self.浏览器.execute_script(js)
        return True

    def 窗口截图保存(self, 保存地址):
        '将当前窗口的屏幕快照保存到PNG图像文件中'
        self.浏览器.save_screenshot(保存地址)
        return True

    def 截图保存(self, 保存地址):
        '用于截取当前窗口，并把图片保存到本地'
        self.浏览器.get_screenshot_as_file(保存地址)
        return True

    def 截图base64(self):
        '以base64编码的字符串获取当前窗口的屏幕快照'
        return self.浏览器.get_screenshot_as_base64()

    def 截图png(self):
        '以二进制数据获取当前窗口的屏幕快照。'
        return self.浏览器.get_screenshot_as_png()

    def 置页面超时时间(self, 秒数):
        '设置等待页面加载完成的时间'
        self.浏览器.set_page_load_timeout(秒数)
        return True

    def 置JS超时时间(self, 秒数):
        '设置脚本在执行过程中应等待的时间'
        self.浏览器.set_script_timeout(秒数)
        return True

    def 置浏览器位置(self, 左边=None, 顶边=None, 宽度=None, 高度=None):
        '设置窗口的x，y坐标以及当前窗口的高度和宽度。'
        self.浏览器.set_window_rect(左边, 顶边, 宽度, 高度)
        return True

    def 鼠标抬起(self):
        '释放按住的鼠标键'
        ActionChains(self.浏览器).release().perform()
        return True


class 浏览器元素操作():

    def __init__(self, 浏览器, 元素):
        self.元素 = 元素
        self.浏览器 = 浏览器

    def 清空(self):
        self.元素.clear()
        return True

    def 输入(self, *args):
        '模拟按键输入，将密钥发送到当前的焦点元素,包括上传文件可以直接发送文件路径,可使用 键盘_* ,如 按键_输入(键盘_CTRL,"a") 即为全选'
        self.元素.send_keys(*args)
        return True

    def 键盘指定输入(self, 键值):
        '将键发送到元素,可以使用 键盘_*'
        ActionChains(self.浏览器).send_keys_to_element(self.元素, 键值).perform()
        return True

    def 点击(self):
        self.元素.click()
        return True

    def 提交表单(self):
        '查找到表单（from）直接调用submit即可'
        self.元素.submit()
        return True

    def 取属性值(self, 属性名称):
        '获取元素属性值,如：value,str,img等属性'
        return self.元素.get_attribute(属性名称)

    def 取CSS属性值(self, 属性名称):
        '获取CSS属性值'
        return self.元素.value_of_css_property(属性名称)

    def 取坐标(self):
        return self.元素.location

    def 取宽度高度(self):
        return self.元素.size

    def 是否显示可见(self):
        '判断元素是否显示'
        return self.元素.is_displayed()

    def 是否选中(self):
        '判断元素是否选中状态'
        return self.元素.is_selected()

    def 是否启用(self):
        '判断元素是否是否启用'
        return self.元素.is_enabled()

    def 取文本(self):
        '获取文本'
        return self.元素.text

    def 取源码(self):
        '获取文本'
        return self.元素.get_attribute("innerHTML")

    def 取标签名称(self):
        '返回元素的tagName'
        return self.元素.tag_name

    def 鼠标悬停(self):
        '执行鼠标悬停操作,一些网页鼠标悬停后才会加载下拉列表'
        ActionChains(self.浏览器).move_to_element(self.元素).perform()
        return True

    def 鼠标右键按下(self):
        ActionChains(self.浏览器).context_click(self.元素).perform()
        return True

    def 鼠标双击(self):
        ActionChains(self.浏览器).double_click(self.元素).perform()
        return True

    def 鼠标左键按住(self):
        '按下鼠标左键在一个元素上'
        ActionChains(self.浏览器).click_and_hold(self.元素).perform()
        return True

    def 鼠标拖动(self, 目标标识):
        '鼠标拖动(d.标识)'
        ActionChains(self.浏览器).drag_and_drop(self.元素, 目标标识).perform()
        return True

    def 鼠标拖动距离(self, X偏移量, Y偏移量):
        '然后移至目标偏移量并释放鼠标按钮'
        ActionChains(self.浏览器).drag_and_drop_by_offset(self.元素, X偏移量, Y偏移量).perform()
        return True

    def 鼠标原地偏移(self, X偏移量, Y偏移量):
        '将鼠标移动到当前鼠标位置的偏移处'
        ActionChains(self.浏览器).move_by_offset(X偏移量, Y偏移量).perform()
        return True

    def 鼠标移到元素中间(self):
        '将鼠标移到元素的中间'
        ActionChains(self.浏览器).move_to_element().perform()
        return True

    def 鼠标移动(self, X偏移量, Y偏移量):
        '将鼠标移动指定元素的偏移量'
        ActionChains(self.浏览器).move_to_element_with_offset(self.元素, X偏移量, Y偏移量).perform()
        return True

    def 鼠标抬起(self):
        '释放元素上按住的鼠标键'
        ActionChains(self.浏览器).release(self.元素).perform()
        return True

    def 按下全选(self):
        self.元素.send_keys(键盘_CTRL, 'a')
        return True

    def 按下复制(self):
        self.元素.send_keys(键盘_CTRL, 'c')
        return True

    def 按下剪切(self):
        self.元素.send_keys(键盘_CTRL, 'x')
        return True

    def 按下粘贴(self):
        self.元素.send_keys(键盘_CTRL, 'v')
        return True

    def 下拉框选择值(self, value):
        'select标签的value属性的值'
        Select(self.元素).select_by_value(value)
        return True

    def 下拉框选择索引(self, id):
        '下拉框的索引'
        Select(self.元素).select_by_index(id)
        return True

    def 下拉框选择文本(self, text):
        '下拉框的文本值'
        Select(self.元素).select_by_visible_text(text)
        return True

    def 下拉框全部取消选中(self):
        Select(self.元素).deselect_all()
        return True

    def 下拉框取消选中索引(self, id):
        Select(self.元素).deselect_by_index(id)
        return True

    def 下拉框取消选中值(self, value):
        Select(self.元素).deselect_by_value(value)
        return True

    def 下拉框取消选中文本(self, text):
        Select(self.元素).deselect_by_visible_text(text)
        return True

    def 下拉框取全部选中项(self):
        '返回属于此选择标记的所有选择的选项的列表'
        return Select(self.元素).all_selected_options

    def 下拉框取选中项(self):
        '该选择标记中的第一个选择的选项（或正常选择中的当前选择的选项）'
        return Select(self.元素).first_selected_option

    def 下拉框取列表项(self):
        '返回属于该选择标签的所有选项的列表'
        return Select(self.元素).options

    def 取矩形(self):
        return self.元素.rect

    def 截图保存(self, 文件名):
        '将当前元素的屏幕快照保存到PNG图像文件中'
        self.元素.screenshot(文件名)
        return True

    def 截图base64(self):
        '以base64编码字符串的形式获取当前元素的屏幕快照'
        return self.元素.screenshot_as_base64

    def 截图png(self):
        '以二进制数据获取当前元素的屏幕截图'
        return self.元素.screenshot_as_png



import unittest

from zip import *
from pyefun import *


class Testezip(unittest.TestCase):

    def test_1(self):
        zip解压("./test.zip", 取运行目录()+"/test2")
    def test_2(self):
        zip压缩(取运行目录()+"/test.zip", 取运行目录()+"/")
    def test_3(self):
        zip压缩("./test单文件.zip", 取运行目录()+"/test/zip.py")



"""

.. Hint::
    进度显示 使您可以测量和打印迭代过程的进度。这可以通过可迭代的界面或使用手动API来完成。使用可迭代的接口是最常见的。


.. literalinclude:: ../../../pyefun/progiterUtil_test.py
    :language: python
    :caption: 代码示例
    :linenos:

"""

from ubelt.util_format import *
import ubelt as ub


def 调试输出(obj):
    # data = repr2(obj, nl=2, precision=2)
    data = repr2(obj)
    print(data)


class 进度显示(ub.ProgIter):
    def __init__(self, 迭代对象=None,
                 描述="",
                 总数=None,
                 信息级别=3,
                 显示速率=False,
                 显示时间=False,
                 起始索引=0,
                 进度大小=None,
                 启用=True,
                 输出在同一行=True
                 ):
        "显示 0 不显示 1结束显示 2概率显示 3全部显示"
        # ProgIter(iterable=迭代对象,total=总数, desc=描述, show_times=False, verbose=显示)
        super().__init__(iterable=迭代对象,
                         total=总数,
                         desc=描述,
                         show_times=显示速率,
                         show_wall=显示时间,
                         verbose=信息级别,
                         initial=起始索引,
                         chunksize=进度大小,
                         enabled=启用,
                         clearline=输出在同一行
                         )


    def 下一步(self, 步数=1, 强制显示=False):
        self.step(inc=步数, force=强制显示)

    def 完成(self, 步数=1, 强制显示=False):
        self.end()

    def 开始(self):
        self.begin()

    def 取进度(self):
        data = self.format_message()
        return data

    def 换行(self):
        self.ensure_newline()

    def 输出(self, obj):
        self.ensure_newline()
        print(obj)

    def 附加输出(self, obj):
        self.set_extra(obj)


"""
定时任务

需要安装第三方模块

pip install apscheduler


"""

from apscheduler.schedulers.blocking import BlockingScheduler
from apscheduler.schedulers.background import BackgroundScheduler

def 时钟_循环间隔模式(后台模式=False, 子程序=None, 元组参数=None, 字典参数=None, 序号=None, weeks周=0, days天=0, hours小时=0,
              minutes分钟=0, seconds秒钟=0, 开始时间='2020-01-01 00:00:01', 结束时间=None, 延时执行=None, 同时允许的线程数=0):
    """

    设置最大同时允许线程数可以解决 程序本身允许时间超过循环的时间,
    例子,scheduler .add_job(job_func, 'interval', minutes=2, start_date='2019-04-15 17:00:00' , end_date='2019-12-31 24:00:00')

    """
    if 同时允许的线程数 == 0:
        if 后台模式 == False:
            scheduler = BlockingScheduler()
        else:
            scheduler = BackgroundScheduler()
    else:
        job_defaults = {'max_instances': 同时允许的线程数}
        if 后台模式 == False:
            scheduler = BlockingScheduler(job_defaults=job_defaults)
        else:
            scheduler = BackgroundScheduler(job_defaults=job_defaults)
    scheduler.add_job(func=子程序, trigger='interval', args=元组参数, kwargs=字典参数, id=序号, weeks=weeks周, days=days天,
                      hours=hours小时,
                      minutes=minutes分钟, seconds=seconds秒钟, start_date=开始时间, end_date=结束时间, jitter=延时执行)
    scheduler.start()

def 时钟_一次性运行(后台模式=False, 子程序=None, 元组参数=None, 字典参数=None, 序号=None, datetime运行的日期=None, 同时允许的线程数=0):
    """

    设置最大同时允许线程数可以解决 程序本身允许时间超过循环的时间,
    运行时间可以是datetime 或 文本型时间 scheduler.add_job(my_job, 'date', run_date='2009-11-06 16:30:05', args=['测试任务'])

    """
    if 同时允许的线程数 == 0:
        if 后台模式 == False:
            scheduler = BlockingScheduler()
        else:
            scheduler = BackgroundScheduler()
    else:
        job_defaults = {'max_instances': 同时允许的线程数}
        if 后台模式 == False:
            scheduler = BlockingScheduler(job_defaults=job_defaults)
        else:
            scheduler = BackgroundScheduler(job_defaults=job_defaults)
    scheduler.add_job(func=子程序, trigger='date', args=元组参数, kwargs=字典参数, id=序号, run_date=datetime运行的日期)
    scheduler.start()




import wx
from .wxControl import *

class 滚动条(wx.ScrollBar, 公用方法):
    pass

    def 取页面大小(self):
        return self.GetPageSize()

    def 取最大位置(self):
        return self.GetRange()

    def 取当前位置(self):
        return self.GetThumbPosition()

    def 取滑块大小(self):
        return self.GetThumbSize()

    def 是否为垂直滚动条(self):
        return self.IsVertical()

    @组件_异常检测
    def 置当前位置(self,位置):
        return self.SetThumbPosition()

    def 置滚动条属性(self,当前位置,滑块大小,最大位置,页面大小,是否重绘=True):
        """
        假设您希望使用相同的字体显示50行文本。窗口的大小设置为一次只能看到16行。您将使用：
        scrollbar.SetScrollbar(0, 16, 50, 15)

        页面大小比拇指大小小1，因此上一页的最后一行将在下一页可见，以帮助定向用户
        """
        return self.SetScrollbar(当前位置,滑块大小,最大位置,页面大小,是否重绘)




import wx
from .wxControl import *

class 分组单选框(wx.RadioBox, 公用方法):
    pass
    @组件_异常检测
    def 启用某项(self,索引):
        return self.EnableItem(索引,True)

    @组件_异常检测
    def 禁用某项(self,索引):
        return self.EnableItem(索引,False)

    @组件_异常检测
    def 置指定项是否可用(self,索引,启用=True):
        return self.EnableItem(索引, 启用)

    @组件_异常检测
    def 查找选项(self,标题文本,区分大小写=False):
        return self.FindString(标题文本,区分大小写)

    def 取选项列数(self):
        '返回单选框中的列数。'
        return self.GetColumnCount()

    def 取选项行数(self):
        return self.GetRowCount()

    def 取选项数(self):
        return self.GetCount()

    def 取指定坐标处选项(self,x,y):
        return self.GetItemFromPoint((x,y))

    @组件_异常检测
    def 取选项帮助文本(self,索引):
        return self.GetItemHelpText(索引)

    @组件_异常检测
    def 取选项文本(self,索引):
        return self.GetItemLabel(索引)

    @组件_异常检测
    def 取选项文本2(self,索引):
        return self.GetString(索引)

    @组件_异常检测
    def 取选项提示工具(self,索引):
        '返回与指定项目关联的工具提示（ 如果有的话）None。返回类型 wx.ToolTip'
        return self.GetItemToolTip()

    @组件_异常检测
    def 取选项提示文本(self,索引):
        return self.GetItemToolTip(索引).GetTip()

    @组件_异常检测
    def 置选项提示文本(self, 索引,提示内容):
        return self.GetItemToolTip(索引).SetTip(提示内容)

    @组件_异常检测
    def 置选项提示时间(self,索引,时间=2000):
        '时间为毫秒'
        return self.GetItemToolTip(索引).SetAutoPop(时间)

    @组件_异常检测
    def 置选项提示延迟显示时间(self, 索引, 时间):
        '时间为毫秒'
        return self.GetItemToolTip(索引).SetDelay(时间)

    @组件_异常检测
    def 置选项提示最大宽度(self, 索引, 宽度):
        '时间为毫秒'
        return self.GetItemToolTip(索引).SetMaxWidth(宽度)

    @组件_异常检测
    def 置选项提示后续提示延时(self,索引,时间):
        '时间为毫秒'
        return self.GetItemToolTip(索引).SetReshow(时间)

    @组件_异常检测
    def 启用或禁用选项提示(self,索引,启用=True):
        return self.GetItemToolTip(索引).Enable(启用)

    def 取现行选中项(self):
        '返回所选项目的索引，或者NOT_FOUND 如果未选择任何项目，则返回该索引 。'
        return self.GetSelection()

    @组件_异常检测
    def 是否启用指定项(self,索引):
        '判断指定选项是否启用,是启用不是选中'
        return self.IsItemEnabled(索引)

    @组件_异常检测
    def 是否显示指定项(self,索引):
        '返回True当前是否显示该项目或False是否使用隐藏该项目Show。'
        return self.IsItemShown(索引)

    @组件_异常检测
    def 置选项帮助文本(self,索引,内容):
        return self.SetItemHelpText(索引,内容)

    @组件_异常检测
    def 置选项标题(self,索引,标题):
        return self.SetItemLabel(索引,标题)

    @组件_异常检测
    def 置选项提示文本(self,索引,提示内容):
        return self.SetItemToolTip(索引,提示内容)

    @组件_异常检测
    def 置选中项(self,索引):
        '指定选项设置为选中状态'
        return self.SetSelection(索引)

    @组件_异常检测
    def 置选项标题(self,索引,标题):
        '指定选项设置为选中状态'
        return self.SetSelection(索引,标题)

    @组件_异常检测
    def 显示某项(self,索引):
        return self.ShowItem(索引,True)

    @组件_异常检测
    def 隐藏某项(self,索引):
        return self.ShowItem(索引,False)

    @组件_异常检测
    def 显示或隐藏某项(self,索引,显示=True):
        return self.ShowItem(索引,显示)



"""

.. Hint::
    二维码编码解码操作

    https://github.com/lincolnloop/python-qrcode

    需要安装

    pip install qrcode

    pip install pyzbar

.. literalinclude:: ../../../pyefun/qrcode/eqrcode_test.py
    :language: python
    :caption: 代码示例
    :linenos:

"""
import qrcode
from PIL import Image
import pyzbar.pyzbar as pyzbar
from io import BytesIO


def 二维码识别(图像字节集):
    pass
    try:
        return str(pyzbar.decode(Image.open(BytesIO(图像字节集)), symbols=[pyzbar.ZBarSymbol.QRCODE])[0][0],
                   encoding="utf-8")
    except:
        return ""


def 二维码生成(编码数据, 优化级别=20, 纠错级别=qrcode.constants.ERROR_CORRECT_M, 格子像素数=10, 边框留白=4):
    """
    error_correction:控制二维码纠错级别。
    ERROR_CORRECT_L:大约7%或者更少的错误会被更正。
    ERROR_CORRECT_M:默认值，大约15%或者更少的错误会被更正。
    ERROR_CORRECT_Q:大约25%或者更少的错误会被更正。
    ERROR_CORRECT_H:大约30%或者更少的错误会被更正。
    box_size:控制二维码中每个格子的像素数，默认为10。
    border:控制二维码四周留白包含的格子数，默认为4。
    image_factory:选择生成图片的形式，默认为PIL图像。
    mask_pattern:选择生成图片的的掩模。

 """
    qr = qrcode.QRCode(
        version=1,
        error_correction=纠错级别,
        box_size=格子像素数,
        border=边框留白)
    qr.add_data(编码数据, optimize=优化级别)
    qr.make()

    im = qr.make_image().get_image()

    img_byte = BytesIO()
    im.save(img_byte, format='png')
    binary_content = img_byte.getvalue()
    img_byte.close()

    return binary_content



from pyefun.arithmeticOperationBase import * # 算数运算
from pyefun.arrayActionBase import * # 数组操作
# from pyefun.codeConv import * # 编码转换
from pyefun.dirkBase import * # 磁盘操作
from pyefun.public import * # 公用模块
from pyefun.stringBase import * # 文本操作
from pyefun.systemProcessingBase import * # 系统处理
from pyefun.timeBase import * # 日期时间操作
from pyefun.typeConv import * # 类型转换
from pyefun.practical import * # 一些实用的函数

# from pyefun.cacheUtil import * # 缓存工具
# from pyefun.commonlyUtil import * # 实用函数
# from pyefun.progiterUtil import * # 进度显示
from pyefun.stringUtil import * # 文本操作实用函数
from pyefun.timeUtil import * # 时间工具类

# from pyefun.regexpUtil import * # 正则表达式

from pyefun.threadingUtil import * # 线程操作有线程池
from pyefun.processPoolUtil import * # 进程池使用方法
from pyefun.networkUtil import *
# from pyefun.configUtil import * # 配置项
# from pyefun.configEnvUtil import * # 环境变量
from pyefun.clockUtil import * # 时钟

# from pyefun.encoding.compress.egzip import *
# from pyefun.encoding.compress.ezlib import *
# from pyefun.encoding.compress.zip import *
# from pyefun.encoding.ebinary.binary import *
# from pyefun.encoding.ebase64.ebase64 import *
# from pyefun.encoding.url.url import *

__version__ = '1.0.27'


import wx.lib.agw.floatspin as floatspin
from .wxControl import *


class 图片操作(wx.Image, 公用方法):
    pass

    def 设置宽度高度(self, width, height, quality=0):
        return 图片操作(self.Scale(width, height, quality))

    def 取位图(self, depth=-1):
        return self.ConvertToBitmap(depth)



#-*- coding: utf-8 -*-
import random,io
import matplotlib.pyplot as plt
import numpy as np
from .public import *


图表颜色 = [
    '#F0F8FF', '#FAEBD7', '#00FFFF', '#7FFFD4', '#F0FFFF', '#F5F5DC', '#FFE4C4', '#FFEBCD', '#8A2BE2', '#A52A2A',
    '#DEB887', '#5F9EA0', '#7FFF00', '#D2691E', '#FF7F50', '#6495ED', '#FFF8DC', '#DC143C', '#00FFFF', '#008B8B',
    '#B8860B', '#A9A9A9', '#006400', '#BDB76B', '#8B008B', '#556B2F', '#FF8C00', '#9932CC', '#8B0000', '#E9967A',
    '#8FBC8F', '#483D8B', '#2F4F4F', '#00CED1', '#9400D3', '#FF1493', '#00BFFF', '#696969', '#1E90FF', '#B22222',
    '#FFFAF0', '#228B22', '#FF00FF', '#DCDCDC', '#F8F8FF', '#FFD700', '#DAA520', '#808080', '#008000', '#ADFF2F',
    '#F0FFF0', '#FF69B4', '#CD5C5C', '#4B0082', '#F0E68C', '#E6E6FA', '#FFF0F5', '#7CFC00', '#FFFACD', '#ADD8E6',
    '#F08080', '#E0FFFF', '#90EE90', '#D3D3D3', '#FFB6C1', '#FFA07A', '#20B2AA', '#87CEFA', '#778899', '#B0C4DE',
    '#00FF00', '#32CD32', '#FAF0E6', '#FF00FF', '#800000', '#66CDAA', '#BA55D3', '#9370DB', '#3CB371', '#7B68EE',
    '#00FA9A', '#48D1CC', '#C71585', '#191970', '#F5FFFA', '#FFE4E1', '#FFE4B5', '#FFDEAD', '#FDF5E6', '#808000',
    '#6B8E23', '#FFA500', '#FF4500', '#DA70D6', '#EEE8AA', '#98FB98', '#AFEEEE', '#DB7093', '#FFEFD5', '#FFDAB9',
    '#CD853F', '#FFC0CB', '#DDA0DD', '#B0E0E6', '#800080', '#FF0000', '#BC8F8F', '#4169E1', '#8B4513', '#FA8072',
    '#FAA460', '#2E8B57', '#FFF5EE', '#A0522D', '#C0C0C0', '#87CEEB', '#6A5ACD', '#708090', '#FFFAFA', '#00FF7F',
    '#4682B4', '#D2B48C', '#008080', '#D8BFD8', '#FF6347', '#40E0D0', '#EE82EE', '#F5DEB3', '#F5F5F5', '#FFFF00',
    '#9ACD32']


class 圆饼图:

    def __init__(self,标签列表=[],数值列表=[],颜色列表=[],间隔列表=[],起始角度=90,数值显示距离=0.6,保留小数位数=2,阴影=False,显示图例=True,宽高=()):
        self.plt = plt
        self.标签列表 = 标签列表
        self.数值列表 = 数值列表
        self.颜色列表 = 颜色列表
        self.间隔列表 = 间隔列表
        self.起始角度 = 起始角度 #逆时针起始角度设置
        self.数值显示位置 = 数值显示距离 #0-1,数值距圆心半径倍数距离
        self.保留小数位数 = 保留小数位数 #数值保留固定小数位
        self.显示阴影 = 阴影 #设置阴影
        self.宽高 = ()
        self.显示图例 = 显示图例

    @异常处理返回类型逻辑型
    def 生成(self,保存地址='',显示=True):
        '返回：图片二进制,标签列表,比例列表'
        fig = self.plt.figure()
        self.plt.rcParams['font.sans-serif'] = ['SimHei']  # 解决中文乱码
        if self.宽高:
            self.plt.figure(figsize=(self.宽高[0],self.宽高[1]))

        if not self.标签列表 and not self.数值列表:
            self.标签列表 = ['张三','李四','王五']
            self.数值列表 = [111,222,333]

        数量 = len(self.标签列表)
        if not self.颜色列表 or len(self.颜色列表)<数量:
            颜色列表 = random.sample(图表颜色, 数量)
        else:
            颜色列表 = self.颜色列表

        if not self.间隔列表 or len(self.间隔列表)<数量:
            self.间隔列表 = [0 for i in range(数量)]

        patches, text1, text2 = self.plt.pie(self.数值列表,
                                             explode=self.间隔列表,
                                             labels=self.标签列表,
                                             colors=颜色列表,
                                             autopct='%.{}f%%'.format(self.保留小数位数),
                                             shadow=self.显示阴影,
                                             startangle=self.起始角度,
                                             pctdistance=self.数值显示位置)
        self.plt.axis('equal')

        if self.显示图例:
            self.plt.legend()

        if 保存地址:
            self.plt.savefig(保存地址)

        if 显示:
            self.plt.show()

        canvas=fig.canvas
        buffer = io.BytesIO()
        canvas.print_png(buffer)
        data = buffer.getvalue()
        buffer.close()

        标签列表=[]
        比例列表=[]
        for x in range(len(text1)):
            标签列表.append(text1[x].get_text())
            比例列表.append(text2[x].get_text())

        return data,标签列表,比例列表



class 柱状图:
    def __init__(self,宽高=(),标题="",横向标题="",纵向标题="",标签列表=[],数值列表=[],名称列表=[],颜色列表=[],柱宽=0.25,标题字体大小=20,显示项目名称=True):
        self.np = np
        self.plt = plt
        self.宽高 = 宽高
        self.标题 = 标题
        self.横向标题 = 横向标题
        self.纵向标题 = 纵向标题
        self.标签列表 = 标签列表
        self.数值列表 = [数值列表] if 数值列表 else []
        self.颜色列表 = 颜色列表
        self.名称列表 = 名称列表
        self.柱宽 = 柱宽
        self.显示项目名称 = 显示项目名称
        self.标题字体大小 = 标题字体大小

    @异常处理返回类型逻辑型
    def 加入新数值列表(self,数值列表):
        self.数值列表.append(数值列表)

    @异常处理返回类型逻辑型
    def 生成(self, 保存地址='', 显示=True):
        self.plt.rcParams['font.sans-serif'] = 'Microsoft YaHei'
        fig = self.plt.figure()
        if not self.标签列表 and not self.数值列表:
            self.标签列表 = ['a', 'b', 'c', 'd', 'e']
            self.数值列表 = [[33, 25, 15, 10, 3],[13, 15, 13, 5, 8]]

        数量 = len(self.标签列表)
        x = self.np.arange(数量)
        if self.宽高:
            self.plt.figure(figsize=(self.宽高[0], self.宽高[1]))

        if not self.名称列表 or len(self.名称列表)<len(self.数值列表):
            self.名称列表 = ['名称'+str(i) for i in range(数量)]

        if not self.颜色列表 or len(self.颜色列表) < 数量:
            颜色列表 = random.sample(图表颜色, 数量)
        else:
            颜色列表 = self.颜色列表

        for i in range(len(self.数值列表)):
            d = {'tick_label':self.标签列表} if i == 0 else {}
            if self.显示项目名称:
                d['label'] = self.名称列表[i]
            self.plt.bar(x+i*self.柱宽, self.数值列表[i], width=self.柱宽, color=颜色列表[i],**d)

            for a, b in zip(x, self.数值列表[i]):
                self.plt.text(a+i*self.柱宽, b + 0.1, b, ha='center', va='bottom')

        self.plt.xticks()
        if self.显示项目名称:
            self.plt.legend(loc="upper right")  # (左.left 右.right)防止label和图像重合显示不出来
        self.plt.ylabel(self.纵向标题)
        self.plt.xlabel(self.横向标题)
        self.plt.title(self.标题,fontdict = {'fontsize':self.标题字体大小})

        if 保存地址:
            self.plt.savefig(保存地址)

        if 显示:
            self.plt.show()
        canvas = fig.canvas
        buffer = io.BytesIO()
        canvas.print_png(buffer)
        data = buffer.getvalue()
        buffer.close()
        return data



class 横向柱状图:
    def __init__(self,宽高=(),标题="",横向标题="",纵向标题="",标签列表=[],数值列表=[],颜色值="",标题字体大小=20,显示项目名称=True):
        self.np = np
        self.plt = plt
        self.宽高 = 宽高
        self.标题 = 标题
        self.横向标题 = 横向标题
        self.纵向标题 = 纵向标题
        self.标签列表 = 标签列表
        self.数值列表 = 数值列表
        self.颜色 = 颜色值
        self.标题字体大小 = 标题字体大小

    @异常处理返回类型逻辑型
    def 生成(self, 保存地址='', 显示=True):
        self.plt.rcParams['font.sans-serif'] = 'Microsoft YaHei'
        fig = self.plt.figure()
        if self.宽高:
            self.plt.figure(figsize=(self.宽高[0], self.宽高[1]))

        if not self.颜色:
            self.颜色 = random.choice(图表颜色)

        if not self.标签列表 and not self.数值列表:
            self.标签列表 = ['a', 'b', 'c', 'd', 'e']
            self.数值列表 = [33, 25, 15, 10, 3]

        fig, ax = self.plt.subplots()
        ax.barh(self.标签列表, self.数值列表, color=self.颜色)
        labels = ax.get_xticklabels()
        self.plt.setp(labels, rotation=0, horizontalalignment='right')

        for a, b in zip(self.标签列表, self.数值列表):
            self.plt.text(b + 1, a, b, ha='center', va='center')

        self.plt.ylabel(self.纵向标题)
        self.plt.xlabel(self.横向标题)
        self.plt.title(self.标题,fontdict = {'fontsize':self.标题字体大小})

        if 保存地址:
            self.plt.savefig(保存地址)

        if 显示:
            self.plt.show()

        canvas = fig.canvas
        buffer = io.BytesIO()
        canvas.print_png(buffer)
        data = buffer.getvalue()
        buffer.close()
        return data


class 重叠柱状图:
    def __init__(self,宽高=(),标题="",横向标题="",纵向标题="",标签列表=[],数值列表=[],名称列表=[],颜色列表=[],柱宽=None,标题字体大小=20,显示项目名称=True):
        self.np = np
        self.plt = plt
        self.宽高 = 宽高
        self.标题 = 标题
        self.横向标题 = 横向标题
        self.纵向标题 = 纵向标题
        self.标签列表 = 标签列表
        self.数值列表 = [数值列表] if 数值列表 else []
        self.颜色列表 = 颜色列表
        self.名称列表 = 名称列表
        self.柱宽 = 柱宽
        self.显示项目名称 = 显示项目名称
        self.标题字体大小 = 标题字体大小

    @异常处理返回类型逻辑型
    def 加入新数值列表(self,数值列表):
        self.数值列表.append(数值列表)

    @异常处理返回类型逻辑型
    def 生成(self, 保存地址='', 显示=True):
        self.plt.rcParams['font.sans-serif'] = 'Microsoft YaHei'
        fig = self.plt.figure()
        if not self.标签列表 and not self.数值列表:
            self.标签列表 = ['a', 'b', 'c', 'd', 'e']
            self.数值列表 = [[33, 25, 15, 10, 9],[13, 15, 13, 5, 8]]

        数量 = len(self.标签列表)
        x = self.np.arange(数量)
        if self.宽高:
            self.plt.figure(figsize=(self.宽高[0], self.宽高[1]))

        if not self.名称列表 or len(self.名称列表)<len(self.数值列表):
            self.名称列表 = ['名称'+str(i) for i in range(数量)]

        if not self.颜色列表 or len(self.颜色列表) < 数量:
            颜色列表 = random.sample(图表颜色, 数量)
        else:
            颜色列表 = self.颜色列表

        for i in range(len(self.数值列表)):
            d = {'tick_label':self.标签列表} if i == 0 else {}
            if self.显示项目名称:
                d['label'] = self.名称列表[i]
            if self.柱宽:
                d['width'] = self.柱宽
            self.plt.bar(self.标签列表, self.数值列表[i], color=颜色列表[i],**d)
            for a, b in zip(x, self.数值列表[i]):
                self.plt.text(a, b + 0.1, b, ha='center', va='bottom')
        self.plt.xticks(self.np.arange(数量), self.标签列表, rotation=0, fontsize=10)

        if self.显示项目名称:
            self.plt.legend(loc="upper right")  # (左.left 右.right)防止label和图像重合显示不出来
        self.plt.ylabel(self.纵向标题)
        self.plt.xlabel(self.横向标题)
        self.plt.title(self.标题,fontdict = {'fontsize':self.标题字体大小})

        if 保存地址:
            self.plt.savefig(保存地址)

        if 显示:
            self.plt.show()

        canvas = fig.canvas
        buffer = io.BytesIO()
        canvas.print_png(buffer)
        data = buffer.getvalue()
        buffer.close()
        return data



class 折线图:
    def __init__(self,宽高=(),标题="",横向标题="",纵向标题="",标签列表=[],数值列表=[],名称列表=[],颜色列表=[],标题字体大小=20,显示项目名称=True,显示数值=True,显示圆点=True):
        self.np = np
        self.plt = plt
        self.宽高 = 宽高
        self.标题 = 标题
        self.横向标题 = 横向标题
        self.纵向标题 = 纵向标题
        self.标签列表 = 标签列表
        self.数值列表 = [数值列表] if 数值列表 else []
        self.颜色列表 = 颜色列表
        self.名称列表 = 名称列表
        self.显示项目名称 = 显示项目名称
        self.标题字体大小 = 标题字体大小
        self.显示数值 = 显示数值
        self.显示圆点 = 显示圆点

    @异常处理返回类型逻辑型
    def 加入新数值列表(self,数值列表):
        self.数值列表.append(数值列表)

    @异常处理返回类型逻辑型
    def 生成(self, 保存地址='', 显示=True):
        self.plt.rcParams['font.sans-serif'] = 'Microsoft YaHei'
        fig = self.plt.figure()
        if not self.标签列表 and not self.数值列表:
            self.标签列表 = ['a', 'b', 'c', 'd', 'e']
            self.数值列表 = [[33, 25, 15, 10, 3],[13, 15, 13, 5, 8]]

        数量 = len(self.标签列表)
        x = self.np.arange(数量)
        if self.宽高:
            self.plt.figure(figsize=(self.宽高[0], self.宽高[1]))

        if not self.名称列表 or len(self.名称列表)<len(self.数值列表):
            self.名称列表 = ['名称'+str(i) for i in range(数量)]

        if not self.颜色列表 or len(self.颜色列表) < 数量:
            颜色列表 = random.sample(图表颜色, 数量)
        else:
            颜色列表 = self.颜色列表

        for i in range(len(self.数值列表)):
            d = {}
            if self.显示项目名称:
                d['label'] = self.名称列表[i]
            if self.显示圆点:
                d['marker'] = '.'

            self.plt.plot(self.标签列表,self.数值列表[i],color=颜色列表[i],linewidth=1,mfc='w',markersize=10,mfcalt='b',**d)

            if self.显示数值:
                for a, b in zip(self.标签列表, self.数值列表[i]):
                    plt.text(a, b, b, ha='center', va='bottom', fontsize=8)

        if self.显示项目名称:
            self.plt.legend(loc="upper right")  # (左.left 右.right)防止label和图像重合显示不出来
        self.plt.ylabel(self.纵向标题)
        self.plt.xlabel(self.横向标题)
        self.plt.title(self.标题,fontdict = {'fontsize':self.标题字体大小})

        #加虚线背景 透明度1 去掉是实线
        self.plt.grid(alpha=1,linestyle=':')

        if 保存地址:
            self.plt.savefig(保存地址)

        if 显示:
            self.plt.show()

        canvas = fig.canvas
        buffer = io.BytesIO()
        canvas.print_png(buffer)
        data = buffer.getvalue()
        buffer.close()
        return data


import unittest

from .javscript import *


class TestJs(unittest.TestCase):

    def test_1(self):
        pass
        js = javscript("JScript")
        data = js.运行("1 + 2")
        self.assertEqual(data, 3)

        js = javscript("Node")
        data = js.运行("1 + 2")
        self.assertEqual(data, 3)

        js = javscript("Node")
        js.加载代码(""" 
        function add(x, y) { 
          return x + y; 
        } 
        """)
        data = js.执行("add", 1, 2)
        self.assertEqual(data, 3)
        data = js.运行("add(1,2)")
        self.assertEqual(data, 3)

        data = 运行js(""" 
        function add(x, y) { 
          return x + y; 
        } 
        """, "add(1,2)")
        self.assertEqual(data, 3)



import wx
from .wxControl import *


class 按钮(wx.Button, 公用方法):
    pass

    @组件_异常检测
    def 置认证图标(self, 显示=True):
        return self.SetAuthNeeded(显示)

    def 显示认证图标(self):
        return self.SetAuthNeeded()

    def 隐藏认证图标(self):
        return self.SetAuthNeeded(False)

    def 置顶层默认项(self):
        '设置后再窗口中按回车即可触发按钮点击事件'
        return self.SetDefault()

    def 文字对齐方式(self, value):
        return self.ToggleWindowStyle(value)

    @property
    def 文字对齐方式(self):
        return ""

    @文字对齐方式.setter
    def 文字对齐方式(self, value):
        self.SetWindowStyle(value)

    @property
    def 图片(self):
        return ""

    @图片.setter
    def 图片(self, 图片路径):
        w, h = self.GetSize()
        img = wx.Image(图片路径).Scale(w, h).ConvertToBitmap()
        self.SetBitmap(img)

    def 置图片(self, Bitmap):
        self.SetBitmap(Bitmap)

class Button(按钮, 公用方法):
    pass



"""

.. Hint::
    磁盘操作

.. literalinclude:: ../../../pyefun/dirkBase.py
    :language: python
    :caption: 代码示例
    :linenos:


"""
import os
import sys
import stat
import shutil
from .public import *
from .stringBase import *


# 取磁盘总空间
# 取磁盘剩余空间
# 取磁盘卷标
# 置磁盘卷标
# 改变驱动器
# 改变目录 -
# 取当前目录 -
# 创建目录 -
# 删除目录 -
# 复制文件 -
# 移动文件 -
# 删除文件 -
# 文件更名 -
# 文件是否存在 -
# 寻找文件
# 取文件时间
# 取文件尺寸
# 取文件属性
# 置文件属性
# 取临时文件名
# 读入文件 -
# 写到文件 -


def 取运行目录():
    return sys.path[0]


def 取当前目录():
    return os.getcwd()


# 调用格式： 〈逻辑型〉 复制文件 （文本型 被复制的文件名，文本型 复制到的文件名） - 系统核心支持库->磁盘操作
# 英文名称：FileCopy
# 成功返回真，失败返回假。本命令为初级命令。
# 参数<1>的名称为“被复制的文件名”，类型为“文本型（text）”。
# 参数<2>的名称为“复制到的文件名”，类型为“文本型（text）”。
#
# 操作系统需求： Windows、Linux
@异常处理返回类型逻辑型
def 复制文件(被复制的文件名, 复制到的文件名):
    shutil.copyfile(被复制的文件名, 复制到的文件名)
    return True


@异常处理返回类型逻辑型
def 复制目录(被复制的目录, 复制到的目录):
    shutil.copytree(被复制的目录, 复制到的目录)
    return True

# 调用格式： 〈逻辑型〉 移动文件 （文本型 被移动的文件，文本型 移动到的位置） - 系统核心支持库->磁盘操作
# 英文名称：FileMove
# 将文件从一个位置移动到另外一个位置。成功返回真，失败返回假。本命令为初级命令。
# 参数<1>的名称为“被移动的文件”，类型为“文本型（text）”。
# 参数<2>的名称为“移动到的位置”，类型为“文本型（text）”。
#
# 操作系统需求： Windows、Linux
@异常处理返回类型逻辑型
def 移动文件(被移动的文件, 移动到的位置):
    shutil.move(被移动的文件, 移动到的位置)
    return True


def 文件_目录文件列表(路径='.'):
    '.为单前目录，..为上级目录，目录下的文件名,文件夹名,不带路径'
    return os.listdir(路径)


def 文件_遍历子目录(路径='.'):
    '成功返回列表：(路径, [包含目录], [包含文件]),用法 for root, dirs, files in os.walk("..", topdown=False):'
    return list(os.walk(路径))


@异常处理返回类型逻辑型
def 创建目录(路径, 自动创建目录=True):
    if 文件是否存在(路径):
        return True
    if 自动创建目录:
        try:
            access = 0o777
            original_umask = os.umask(000)
            os.makedirs(路径, access)
        finally:
            os.umask(original_umask)
    else:
        os.mkdir(路径)
    return True


# 调用格式： 〈逻辑型〉 写到文件 （文本型 文件名，字节集 欲写入文件的数据，... ） - 系统核心支持库->磁盘操作
# 英文名称：WriteFile
# 本命令用作将一个或数个字节集顺序写到指定文件中，文件原有内容被覆盖。成功返回真，失败返回假。本命令为初级命令。命令参数表中最后一个参数可以被重复添加。
# 参数<1>的名称为“文件名”，类型为“文本型（text）”。
# 参数<2>的名称为“欲写入文件的数据”，类型为“字节集（bin）”。
#
# 操作系统需求： Windows、Linux

@异常处理返回类型逻辑型
def 写到文件(文件名, 欲写入文件的数据):
    if (type(欲写入文件的数据) == str):
        欲写入文件的数据 = bytes(欲写入文件的数据, encoding="utf-8")

    with open(文件名, 'wb') as f:
        f.write(欲写入文件的数据)
    return True


def 读入文件(文件名):
    if 文件是否存在(文件名) == False:
        return b""
    '方式默认用r 字节集用rb 长度默认读取全部,更多模式参考https://www.runoob.com/python3/python3-func-open.html'
    with open(文件名, 'rb') as f:
        return f.read(-1)


@异常处理返回类型逻辑型
def 删除文件(路径):
    '成功返回True,用于删除文件,如果文件是一个目录则返回一个错误'
    os.remove(路径)
    return True


@异常处理返回类型逻辑型
def 删除目录(路径, 递归删除=False):
    if 文件是否存在(路径) == False:
        return False
    if 递归删除:
        shutil.rmtree(路径)
    else:
        os.rmdir(路径)
    return True


@异常处理返回类型逻辑型
def 文件更名(原文件名, 新文件名):
    '成功返回True,可以是文件或文件夹'
    # os.rename(原文件名, 新文件名)
    shutil.move(原文件名, 新文件名)

    return True


def 文件_路径取扩展名(路径):
    return os.path.splitext(路径)[1]


def 文件_取文件名(路径):
    return os.path.basename(路径)


def 路径_取目录名(路径):
    return os.path.basename(路径)


def 文件_取目录(路径):
    '去掉文件名，返回目录路径'
    return os.path.dirname(路径)


@异常处理返回类型逻辑型
def 改变目录(路径):
    '成功返回True'
    os.chdir(路径)
    return True


@异常处理返回类型逻辑型
def 改变当前进程目录(路径):
    '成功返回True'
    os.chroot(路径)
    return True


def 文件_检查权限(路径, 权限=os.F_OK):
    '类型：0 是否存在 1 是否可读 2 是否可写 3 是否可执行，返回True或False'
    # {0: os.F_OK, 1: os.R_OK, 2: os.W_OK, 3: os.X_OK}
    return os.access(路径, 权限)


def 文件_是否为绝对路径(路径):
    '传入路径返回True或False'
    return os.path.isabs(路径)


def 文件_是否为目录(路径):
    '传入路径返回True或False'
    return os.path.isdir(路径)


def 文件_是否为文件(路径):
    '传入路径返回True或False'
    return os.path.isfile(路径)


def 文件是否存在(路径):
    '传入路径返回True或False'
    return os.path.exists(路径)


def 文件_取文件大小(路径):
    '返回文件长度'
    return os.path.getsize(路径)


def 文件_获取文件信息(路径):
    '成功返回(上次访问时间,修改时间,文件大小)，返回的是10位时间戳'
    结果 = os.stat(路径)
    return (结果.st_atime, 结果.st_mtime, 结果.st_size)


def 文件_修改文件时间(路径, 时间):
    '成功返回True,传入的时间为10位时间戳元组类型(访问时间戳,修改时间戳)'
    os.utime(路径, 时间)
    return True


# 调用格式： 〈日期时间型〉 取文件时间 （文本型 文件名） - 系统核心支持库->磁盘操作
# 英文名称：FileDateTime
# 返回指定文件被创建或最后修改后的日期和时间。如果该文件不存在，将返回100年1月1日。本命令为初级命令。
# 参数<1>的名称为“文件名”，类型为“文本型（text）”。
#
# 操作系统需求： Windows、Linux
def 取文件访问时间(路径):
    '返回时间戳'
    return os.path.getatime(路径)


def 取文件创建时间(路径):
    '返回时间戳'
    return os.path.getctime(路径)


def 取文件修改时间(路径):
    '返回时间戳'
    return os.path.getmtime(路径)


def 文件_修改权限(路径, 权限=stat.S_IRWXU):
    'http://www.runoob.com/python/os-chmod.html'
    # stat.S_IREAD 只读
    # stat.S_IWRITE 取消只读
    # stat.S_IRWXU 读写执行
    return os.chmod(路径, 权限)


def 文件_枚举(欲寻找的目录=".", name=".jpg", 递归子目录=True):
    list = []
    for item in os.listdir(欲寻找的目录):
        item_path = os.path.join(欲寻找的目录, item)
        if os.path.isdir(item_path):
            if (递归子目录):
                newlist = 文件_枚举(item_path, name, 递归子目录)
                for item_path in newlist:
                    list.append(item_path)
        elif os.path.isfile(item_path):
            if name in item:
                list.append(item_path)
    return list


def 目录_枚举(欲寻找的目录=".", 递归子目录=True):
    list = []
    for item in os.listdir(欲寻找的目录):
        item_path = os.path.join(欲寻找的目录, item)
        if os.path.isdir(item_path):
            list.append(item_path)
            if (递归子目录):
                newlist = 目录_枚举(item_path, 递归子目录)
                for item_path in newlist:
                    list.append(item_path)
    return list


def 文件_删除(欲删除的文件名: str) -> bool:
    return 删除文件(欲删除的文件名)


def 文件_取扩展名(路径: str) -> str:
    return 子文本替换(文件_路径取扩展名(路径), ".", "")


def 文件_更名(原文件名, 新文件名) -> bool:
    return 文件更名(原文件名, 新文件名)


def 文件_取父目录(路径) -> str:
    return os.path.dirname(路径)


def 文件_写出(文件名: str, 欲写入文件的数据) -> bool:
    dir = 文件_取父目录(文件名)
    if 文件是否存在(dir) == False:
        创建目录(dir)
    return 写到文件(文件名, 欲写入文件的数据)


@异常处理返回类型逻辑型
def 文件_追加文本(文件名: str, 欲追加的文本: str) -> bool:
    dir = 文件_取父目录(文件名)
    if 文件是否存在(dir) == False:
        创建目录(dir)
    with open(文件名, "a") as f:
        f.write(str(欲追加的文本) + "\n")
    return True


@异常处理返回类型逻辑型
def 读入文本(文件名: str) -> str:
    return 读入文件(文件名).decode('utf-8')


def 文件_保存(文件名: str, 欲写入文件的数据: bytes) -> str:
    dir = 文件_取父目录(文件名)
    if 文件是否存在(dir) == False:
        创建目录(dir)
        return 文件_写出(文件名, 欲写入文件的数据)
    else:
        data = 读入文件(文件名)
        wdata = 欲写入文件的数据
        if (data != wdata):
            return 文件_写出(文件名, 欲写入文件的数据)
    return True


def 文件从列表中选取存在的文件路径(路径列表):
    """
    匹配文件是否存在 存在则返回文件路径
    
.. code-block:: python
   :linenos:

    labelsPath = 文件从列表中选取存在的文件路径([
        "./model/labels.txt",
        "/opt/labels.txt",
        "/opt/model/labels.txt",
    ])
    """
    for v in 路径列表:
        if os.path.exists(v):
            return v
    return ''


def 路径优化(path):
    """
    把\\ // 乱七八糟的路径转化为规整的
    :param path:
    :return:
    """
    if 寻找文本(path, "\\") > -1:
        path = 子文本替换(path, "\\", "/")
    if 寻找文本(path, r"\\") > -1:
        path = 子文本替换(path, r"\\", "/")
    if 寻找文本(path, r"//") > -1:
        path = 子文本替换(path, r"//", "/")
    return os.path.normpath(path)


import uuid
import hashlib
import random
from .dirkBase import *


def 取uuid():
    return str(uuid.uuid4())

def 取短id():
    array = ["a", "b", "c", "d", "e", "f",
             "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "q", "r", "s",
             "t", "u", "v", "w", "x", "y", "z", "0", "1", "2", "3", "4", "5",
             "6", "7", "8", "9", "A", "B", "C", "D", "E", "F", "G", "H", "I",
             "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V",
             "W", "X", "Y", "Z"]
    id = str(uuid.uuid4()).replace("-", '')
    buffer = []

    for i in range(0, 8):
        start = i * 4
        end = i * 4 + 4
        val = int(id[start:end], 16)
        buffer.append(array[val % 62])
    return "".join(buffer)

def 取md5(内容, 编码="utf-8"):
    MD5 = hashlib.md5()
    MD5.update(内容.encode(encoding=编码))
    return MD5.hexdigest()

def 数组_随机排序(items):
    return random.shuffle(items)


def 取执行文件名():
    """
    调用格式： 〈文本型〉 取执行文件名 （） - 系统核心支持库->环境存取

    英文名称：GetRunFileName

    取当前被执行的易程序文件的名称。本命令为初级命令。

    操作系统需求： Windows

    :return: commonlyUtil.py
    """
    return 文件_取文件名(__file__)


def 读环境变量(环境变量名称: str) -> str:
    """
        调用格式： 〈文本型〉 读环境变量 （文本型 环境变量名称） - 系统核心支持库->环境存取

        英文名称：GetEnv

        返回文本，它关连于一个操作系统环境变量。成功时返回所取得的值，失败则返回空文本。本命令为初级命令。

        参数<1>的名称为“环境变量名称”，类型为“文本型（text）”。

        操作系统需求： Windows、Linux

    """
    return os.environ.get(环境变量名称)


def 写环境变量(环境变量名称: str, 欲写入内容: str) -> bool:
    """
    调用格式： 〈逻辑型〉 写环境变量 （文本型 环境变量名称，文本型 欲写入内容） - 系统核心支持库->环境存取

    英文名称：PutEnv

    修改或建立指定的操作系统环境变量。成功返回真，失败返回假。本命令为初级命令。

    参数<1>的名称为“环境变量名称”，类型为“文本型（text）”。

    参数<2>的名称为“欲写入内容”，类型为“文本型（text）”。

    操作系统需求： Windows、Linux


    """
    return os.environ.setdefault(环境变量名称, 欲写入内容)


import socket


def 取主机名():
    """
    调用格式： 〈文本型〉 取主机名 （） - 系统核心支持库->网络通信

    英文名称：GetHostName

    返回本机的主机名，用作在网络通讯中标志本机地址。本命令为初级命令。

    操作系统需求： Windows

    :return:

    """
    return socket.gethostname()


def 转换为IP地址(欲转换主机名):
    """
    调用格式： 〈文本型〉 转换为IP地址 （文本型 欲转换主机名） - 系统核心支持库->网络通信

    英文名称：HostNameToIP

    将指定的主机名转换为其 IP 地址。如果失败返回空文本。本命令为初级命令。

    参数<1>的名称为“欲转换主机名”，类型为“文本型（text）”。

    操作系统需求： Windows

    """
    return socket.gethostbyname(欲转换主机名)


def 标准输入():
    """
    调用格式： 〈文本型〉 标准输入 （［逻辑型 是否回显］） - 系统核心支持库->控制台操作

    英文名称：fgets

    在标准输入设备上请求输入最多包含2048个字符的一行文本，返回用户所输入的内容。注意本命令只能在控制台程序中使用。本命令为初级命令。

    参数<1>的名称为“是否回显”，类型为“逻辑型（bool）”，可以被省略。本参数决定输入时是否显示所输入字符，为假不显示，为真显示。如果被省略，默认值为真，即回显。可以通过将本参数设置为假以输入密码等特殊信息。

    操作系统需求： Windows、Linux


    """
    return input()


def 标准输出(*args):
    """
    调用格式： 〈无返回值〉 标准输出 （［整数型 输出方向］，通用型 欲输出内容，... ） - 系统核心支持库->控制台操作

    英文名称：fputs

    在标准输出设备或标准错误设备上输出指定的内容，注意本命令只能在控制台程序中使用。本命令为初级命令。命令参数表中最后一个参数可以被重复添加。

    参数<1>的名称为“输出方向”，类型为“整数型（int）”，可以被省略。本参数提供内容所输出到的设备，可以为以下常量值之一： 1、#标准输出设备； 2、#标准错误设备。如果省略本参数，默认为“#标准输出设备”。

    参数<2>的名称为“欲输出内容”，类型为“通用型（all）”。本参数只能为文本、数值、逻辑值或日期时间。如果内容为文本且包含多行，可在各行之间用回车符 (即“字符 (13)”)、换行符 (即“字符 (10)”) 或回车换行符的组合 (即：“字符 (13) + 字符 (10)”) 来分隔。

    操作系统需求： Windows、Linux

    """
    return print(*args)


def 结束(*args, **kwargs):
    """
    调用格式： 〈无返回值〉 结束 （） - 系统核心支持库->流程控制

    英文名称：end

    本命令结束当前易程序的运行。本命令为初级命令。

    操作系统需求： Windows、Linux、Unix
    """
    return exit(args, **kwargs)


def 运行python代码(代码,全局变量=None,局部变量=None):
    '动态执行python代码并返回值'
    return eval(代码,全局变量,局部变量)

def 执行python代码(代码,全局变量=None,局部变量=None):
    '动态执行python代码,只返回None'
    return exec(代码,全局变量,局部变量)


def 目录_取文件夹大小(path):
    '传入路径是文件夹'
    size = 0.0
    for root, dirs, files in os.walk(path):
        size += sum([os.path.getsize(os.path.join(root, file)) for file in files])
    size = round(size / 1024 / 1024, 2)
    if size > 1000:
        size = round(size / 1024, 2)
        return str(size)+'-GB'
    return str(size)+'-MB'

def 取桌面目录():
    '返回当前电脑的桌面路径'
    return os.path.join(os.path.expanduser("~"),'Desktop')




import wx
from .wxControl import *
from .Image import *


class 图片框(wx.StaticBitmap, 公用方法):
    pass

    _图片 = None
    _显示方式 = "缩放图片"

    @property
    def 图片(self):
        return self.取图片()

    @图片.setter
    def 图片(self, value):
        self._图片 = value
        if type(value) == str:
            value = wx.Image(value)
            if self._显示方式 == "图片居左上":
                pass
            elif self._显示方式 == "缩放图片":
                pass
                w, h = self.GetSize()
                value = value.Scale(w, h)

            elif self._显示方式 == "图片居中":
                pass

            value = value.ConvertToBitmap()

        self.置图片(value)
        self.Refresh()

    @property
    def 显示方式(self):
        return self._显示方式

    @显示方式.setter
    def 显示方式(self, value):
        self._显示方式 = value
        self.图片 = self._图片

    @property
    def 图标(self):
        return self.取图标()

    @图标.setter
    def 图标(self, value):
        return self.置图标(value)

    def 取图片(self):
        '返回控件中当前使用的位图。'
        return self.GetBitmap()

    def 取图标(self):
        '返回控件中当前使用的图标。'
        return self.GetIcon()

    def 取缩放模式(self):
        '模式： 0.以原始大小显示 1.比例填充  2.通过保持纵横比，缩放位图以适合控件的大小。  3.缩放位图以填充控件的大小。'
        return self.GetScaleMode()

    @组件_异常检测
    def 置图片(self, 图片):
        return self.SetBitmap(图片)

    @组件_异常检测
    def 置图标(self, 图标):
        return self.SetIcon(图标)

    @组件_异常检测
    def 置缩放模式(self, 模式):
        '模式： 0.以原始大小显示 1.比例填充  2.通过保持纵横比，缩放位图以适合控件的大小。  3.缩放位图以填充控件的大小。'
        return self.SetScaleMode(模式)



"""
********
组件的事件
********


- 事件.按钮_被点击 = wx.EVT_BUTTON
- 事件.复选框_被点击 = wx.EVT_CHECKBOX
- 事件.选择_被点击 = wx.EVT_CHOICE
- 事件.列表框_被点击 = wx.EVT_LISTBOX
- 事件.列表框_被双击 = wx.EVT_LISTBOX_DCLICK
- 事件.菜单_被点击 = wx.EVT_MENU
- 事件.菜单范围_被点击 = wx.EVT_MENU_RANGE
- 事件.滑块_被点击 = wx.EVT_SLIDER
- 事件.无线电箱_被点击 = wx.EVT_RADIOBOX
- 事件.单选按钮_被点击 = wx.EVT_RADIOBUTTON

- 事件.鼠标左键按下 = wx.EVT_LEFT_DOWN
- 事件.鼠标左键松开 = wx.EVT_LEFT_UP
- 事件.鼠标左键双击 = wx.EVT_LEFT_DCLICK
- 事件.鼠标右键按下 = wx.EVT_RIGHT_DOWN
- 事件.鼠标右键松开 = wx.EVT_RIGHT_UP
- 事件.鼠标右键双击 = wx.EVT_RIGHT_DCLICK
- 事件.鼠标中键按下 = wx.EVT_MIDDLE_DOWN
- 事件.鼠标中键松开 = wx.EVT_MIDDLE_UP
- 事件.鼠标中键双击 = wx.EVT_MIDDLE_DCLICK
- 事件.鼠标中键滚动 = wx.EVT_MOUSEWHEEL
- 事件.鼠标移动 = wx.EVT_MOTION
- 事件.鼠标进入 = wx.EVT_ENTER_WINDOW
- 事件.鼠标离开 = wx.EVT_LEAVE_WINDOW
- 事件.点击某键 = wx.EVT_CHAR_HOOK
- 事件.按下某键 = wx.EVT_KEY_DOWN
- 事件.松开某键 = wx.EVT_KEY_UP
- 事件.获得焦点 = wx.EVT_SET_FOCUS
- 事件.失去焦点 = wx.EVT_KILL_FOCUS
- 事件.按钮被单击 = wx.EVT_BUTTON
- 事件.创建完毕 = wx.EVT_WINDOW_CREATE
- 事件.尺寸被改变 = wx.EVT_SIZE
- 事件.位置被移动 = wx.EVT_MOVE
- 事件.将被关闭 = wx.EVT_CLOSE
- 事件.内容被改变 = wx.EVT_TEXT
- 事件.按下Enter键 = wx.EVT_TEXT_ENTER
- 事件.达到限制长度 = wx.EVT_TEXT_MAXLEN
- 事件.状态被改变 = wx.EVT_RADIOBUTTON
- 事件.狀态被改变 = wx.EVT_CHECKBOX
- 事件.选中列表项 = wx.EVT_COMBOBOX
- 事件.文本发生变化 = wx.EVT_TEXT
- 事件.弹出列表项 = wx.EVT_COMBOBOX_DROPDOWN
- 事件.收起列表项 = wx.EVT_COMBOBOX_CLOSEUP
- 事件.被拖动 = wx.EVT_SLIDER
- 事件.向小拖动 = wx.EVT_SCROLL_PAGEUP
- 事件.向大拖动 = wx.EVT_SCROLL_PAGEDOWN
- 事件.正在拖动 = wx.EVT_SCROLL_THUMBTRACK
- 事件.停止拖动 = wx.EVT_SCROLL_THUMBRELEASE
- 事件.数值被调整 = wx.EVT_SPINCTRL
- 事件.周期事件 = wx.EVT_TIMER
- 事件.表项被单击 = wx.EVT_CHECKLISTBOX
- 事件.表项被双击 = wx.EVT_LISTBOX_DCLICK
- 事件.链接被单击 = wx.adv.EVT_HYPERLINK
- 事件.双击某一天 = wx.adv.EVT_CALENDAR
- 事件.日期已更改 = wx.adv.EVT_CALENDAR_SEL_CHANGED
- 事件.年月已更改 = wx.adv.EVT_CALENDAR_PAGE_CHANGED
- 事件.内容被更改 = wx.adv.EVT_DATE_CHANGED
- 事件.内容被修改 = wx.adv.EVT_TIME_CHANGED
- 事件.左键开始拖动 = wx.EVT_LIST_BEGIN_DRAG
- 事件.右键开始拖动 = wx.EVT_LIST_BEGIN_RDRAG
- 事件.开始编辑标签 = wx.EVT_LIST_BEGIN_LABEL_EDIT
- 事件.编辑标签结束 = wx.EVT_LIST_END_LABEL_EDIT
- 事件.表项被删除 = wx.EVT_LIST_DELETE_ITEM
- 事件.表项全部被删 = wx.EVT_LIST_DELETE_ALL_ITEMS
- 事件.选中表项 = wx.EVT_LIST_ITEM_SELECTED
- 事件.取消选中表项 = wx.EVT_LIST_ITEM_DESELECTED
- 事件.表项已激活 = wx.EVT_LIST_ITEM_ACTIVATED
- 事件.中键单击表项 = wx.EVT_LIST_ITEM_MIDDLE_CLICK
- 事件.右键单击表项 = wx.EVT_LIST_ITEM_RIGHT_CLICK
- 事件.表项按下某键 = wx.EVT_LIST_KEY_DOWN
- 事件.插入新表项 = wx.EVT_LIST_INSERT_ITEM
- 事件.左键单击某列 = wx.EVT_LIST_COL_CLICK
- 事件.右键单击某列 = wx.EVT_LIST_COL_RIGHT_CLICK
- 事件.开始调整列宽 = wx.EVT_LIST_COL_BEGIN_DRAG
- 事件.分隔线在拖动 = wx.EVT_LIST_COL_DRAGGING
- 事件.列宽被修改 = wx.EVT_LIST_COL_END_DRAG
- 事件.表项被选中 = wx.EVT_LIST_ITEM_CHECKED
- 事件.表项未选中 = wx.EVT_LIST_ITEM_UNCHECKED
- 事件.滚动事件 = wx.EVT_SCROLL
- 事件.滚到最小位置 = wx.EVT_SCROLL_TOP
- 事件.滚到最大位置 = wx.EVT_SCROLL_BOTTOM
- 事件.向小滚动 = wx.EVT_SCROLL_PAGEUP
- 事件.向大滚动 = wx.EVT_SCROLL_PAGEDOWN
- 事件.停止滚动 = wx.EVT_SCROLL_CHANGED
- 事件.选项被单击 = wx.EVT_RADIOBOX
- 事件.颜色被更改 = wx.EVT_COLOURPICKER_CHANGED
- 事件.点击某颜色 = wx.EVT_COLOURPICKER_CURRENT_CHANGED
- 事件.取消更改颜色 = wx.EVT_COLOURPICKER_DIALOG_CANCELLED
- 事件.点击链接 = wx.EVT_TEXT_URL
- 事件.响应左键事件 = hyperlink.EVT_HYPERLINK_LEFT
- 事件.响应中键事件 = hyperlink.EVT_HYPERLINK_MIDDLE
- 事件.响应右键事件 = hyperlink.EVT_HYPERLINK_RIGHT
- 事件.数字被调整 = floatspin.EVT_FLOATSPIN

"""
import wx.lib.agw.floatspin as floatspin
import wx.lib.agw.hyperlink as hyperlink

import wx
import wx.adv
from wx import *


class 事件:
    被单击 = wx.EVT_LEFT_DOWN

    按钮被点击 = wx.EVT_BUTTON
    复选框被点击 = wx.EVT_CHECKBOX
    选择被点击 = wx.EVT_CHOICE
    列表框被点击 = wx.EVT_LISTBOX
    列表框被双击 = wx.EVT_LISTBOX_DCLICK
    菜单被点击 = wx.EVT_MENU
    菜单范围被点击 = wx.EVT_MENU_RANGE
    滑块被点击 = wx.EVT_SLIDER
    单选框被点击 = wx.EVT_RADIOBOX
    单选框按钮被点击 = wx.EVT_RADIOBUTTON

    鼠标左键被按下 = wx.EVT_LEFT_DOWN
    鼠标左键被放开 = wx.EVT_LEFT_UP
    鼠标左键被双击 = wx.EVT_LEFT_DCLICK
    被双击 = wx.EVT_LEFT_DCLICK
    鼠标右键被按下 = wx.EVT_RIGHT_DOWN
    鼠标右键被放开 = wx.EVT_RIGHT_UP
    鼠标右键被双击 = wx.EVT_RIGHT_DCLICK
    鼠标中键被按下 = wx.EVT_MIDDLE_DOWN
    鼠标中键被松开 = wx.EVT_MIDDLE_UP
    鼠标中键被双击 = wx.EVT_MIDDLE_DCLICK
    鼠标中键被滚动 = wx.EVT_MOUSEWHEEL
    滚轮被滚动 = wx.EVT_MOUSEWHEEL
    鼠标位置被移动 = wx.EVT_MOTION
    鼠标进入 = wx.EVT_ENTER_WINDOW
    鼠标离开 = wx.EVT_LEAVE_WINDOW
    点击某键 = wx.EVT_CHAR_HOOK
    按下某键 = wx.EVT_KEY_DOWN
    字符输入 = wx.EVT_KEY_DOWN  # 可能有误
    放开某键 = wx.EVT_KEY_UP
    获得焦点 = wx.EVT_SET_FOCUS
    失去焦点 = wx.EVT_KILL_FOCUS
    按钮被单击 = wx.EVT_BUTTON
    创建完毕 = wx.EVT_WINDOW_CREATE
    尺寸被改变 = wx.EVT_SIZE
    位置被改变 = wx.EVT_MOVE
    可否被关闭 = wx.EVT_CLOSE
    窗口可否被关闭 = wx.EVT_CLOSE
    将被销毁 = wx.EVT_CLOSE
    被激活 = EVT_ACTIVATE
    被取消激活 = EVT_ACTIVATE
    空闲 = EVT_ACTIVATE  # 不知道
    首次激活 = EVT_ACTIVATE
    托盘事件 = EVT_MENU
    被显示 = EVT_SHOW
    被隐藏 = EVT_SHOW  # 不知道

    内容被改变 = wx.EVT_TEXT
    按下Enter键 = wx.EVT_TEXT_ENTER
    达到限制长度 = wx.EVT_TEXT_MAXLEN
    状态被改变 = wx.EVT_RADIOBUTTON
    狀态被改变 = wx.EVT_CHECKBOX
    选中列表项 = wx.EVT_COMBOBOX
    列表项被选择 = wx.EVT_COMBOBOX
    文本发生变化 = wx.EVT_TEXT
    弹出列表项 = wx.EVT_COMBOBOX_DROPDOWN
    收起列表项 = wx.EVT_COMBOBOX_CLOSEUP
    被拖动 = wx.EVT_SLIDER
    向小拖动 = wx.EVT_SCROLL_PAGEUP
    向大拖动 = wx.EVT_SCROLL_PAGEDOWN
    正在拖动 = wx.EVT_SCROLL_THUMBTRACK
    停止拖动 = wx.EVT_SCROLL_THUMBRELEASE
    数值被调整 = wx.EVT_SPINCTRL
    周期事件 = wx.EVT_TIMER
    表项被单击 = wx.EVT_CHECKLISTBOX
    表项被双击 = wx.EVT_LISTBOX_DCLICK
    链接被单击 = wx.adv.EVT_HYPERLINK
    双击某一天 = wx.adv.EVT_CALENDAR
    日期已更改 = wx.adv.EVT_CALENDAR_SEL_CHANGED
    年月已更改 = wx.adv.EVT_CALENDAR_PAGE_CHANGED
    内容被更改 = wx.adv.EVT_DATE_CHANGED
    内容被修改 = wx.adv.EVT_TIME_CHANGED
    左键开始拖动 = wx.EVT_LIST_BEGIN_DRAG
    右键开始拖动 = wx.EVT_LIST_BEGIN_RDRAG
    开始编辑标签 = wx.EVT_LIST_BEGIN_LABEL_EDIT
    编辑标签结束 = wx.EVT_LIST_END_LABEL_EDIT
    表项被删除 = wx.EVT_LIST_DELETE_ITEM
    表项全部被删 = wx.EVT_LIST_DELETE_ALL_ITEMS
    # 超级列表框
    当前表项被改变 = wx.EVT_LIST_ITEM_SELECTED
    表项被激活 = wx.EVT_LIST_ITEM_ACTIVATED
    表头被单击 = wx.EVT_LIST_COL_CLICK
    表项跟踪 = wx.EVT_LIST_COL_CLICK
    左键单击表项 = wx.EVT_LIST_COL_CLICK
    右键单击表项 = wx.EVT_LIST_ITEM_RIGHT_CLICK
    开始编辑 = wx.EVT_LIST_BEGIN_LABEL_EDIT
    结束编辑 = wx.EVT_LIST_END_LABEL_EDIT
    检查框状态被改变 = wx.EVT_LIST_COL_CLICK
    # todo 有很多都不知道是啥 待补充
    选中表项 = wx.EVT_LIST_ITEM_SELECTED
    取消选中表项 = wx.EVT_LIST_ITEM_DESELECTED
    表项已激活 = wx.EVT_LIST_ITEM_ACTIVATED
    中键单击表项 = wx.EVT_LIST_ITEM_MIDDLE_CLICK
    右键单击表项 = wx.EVT_LIST_ITEM_RIGHT_CLICK
    表项按下某键 = wx.EVT_LIST_KEY_DOWN
    插入新表项 = wx.EVT_LIST_INSERT_ITEM
    左键单击某列 = wx.EVT_LIST_COL_CLICK
    右键单击某列 = wx.EVT_LIST_COL_RIGHT_CLICK
    开始调整列宽 = wx.EVT_LIST_COL_BEGIN_DRAG
    分隔线在拖动 = wx.EVT_LIST_COL_DRAGGING
    列宽被修改 = wx.EVT_LIST_COL_END_DRAG
    表项被选中 = wx.EVT_LIST_ITEM_CHECKED
    表项未选中 = wx.EVT_LIST_ITEM_UNCHECKED
    滚动事件 = wx.EVT_SCROLL
    滚到最小位置 = wx.EVT_SCROLL_TOP
    滚到最大位置 = wx.EVT_SCROLL_BOTTOM
    向小滚动 = wx.EVT_SCROLL_PAGEUP
    向大滚动 = wx.EVT_SCROLL_PAGEDOWN
    停止滚动 = wx.EVT_SCROLL_CHANGED
    选项被单击 = wx.EVT_RADIOBOX
    颜色被更改 = wx.EVT_COLOURPICKER_CHANGED
    点击某颜色 = wx.EVT_COLOURPICKER_CURRENT_CHANGED
    取消更改颜色 = wx.EVT_COLOURPICKER_DIALOG_CANCELLED
    点击链接 = wx.EVT_TEXT_URL
    响应左键事件 = hyperlink.EVT_HYPERLINK_LEFT
    响应中键事件 = hyperlink.EVT_HYPERLINK_MIDDLE
    响应右键事件 = hyperlink.EVT_HYPERLINK_RIGHT
    数字被调整 = floatspin.EVT_FLOATSPIN


# 鼠标指针
# https://wxpython.org/Phoenix/docs/html/wx.StockCursor.enumeration.html
class 鼠标指针:
    无 = wx.CURSOR_NONE
    最大 = wx.CURSOR_MAX
    默认型 = wx.CURSOR_ARROW
    标准箭头型 = wx.CURSOR_ARROW
    指向右侧的箭头 = wx.CURSOR_RIGHT_ARROW
    靶心 = wx.CURSOR_BULLSEYE
    矩形字符 = wx.CURSOR_CHAR
    十字型 = wx.CURSOR_CROSS
    手型 = wx.CURSOR_HAND
    文本编辑型 = wx.CURSOR_IBEAM  # 工字梁垂直线
    表示鼠标左键按下 = wx.CURSOR_LEFT_BUTTON
    放大镜 = wx.CURSOR_MAGNIFIER
    表示按下中间按钮的鼠标 = wx.CURSOR_MIDDLE_BUTTON
    禁止符型 = wx.CURSOR_NO_ENTRY
    画笔 = wx.CURSOR_PAINT_BRUSH
    铅笔 = wx.CURSOR_PENCIL
    向上箭头 = wx.CURSOR_ARROW  # 这个好像没有啊
    指向左的 = wx.CURSOR_POINT_LEFT
    指向右的 = wx.CURSOR_POINT_RIGHT
    箭头及问号型 = wx.CURSOR_QUESTION_ARROW
    表示按下右键的鼠标 = wx.CURSOR_RIGHT_BUTTON
    四向箭头型 = wx.CURSOR_SIZENESW
    北东箭头型 = wx.CURSOR_SIZENESW  # 这个好像没有啊
    北南箭头型 = wx.CURSOR_SIZENS
    北西箭头型 = wx.CURSOR_SIZENWSE
    西东箭头型 = wx.CURSOR_SIZEWE
    一般大小的游标 = wx.CURSOR_SIZING
    Spraycan游标 = wx.CURSOR_SPRAYCAN
    沙漏型 = wx.CURSOR_WAIT
    监视沙漏 = wx.CURSOR_WATCH
    透明 = wx.CURSOR_BLANK
    标准X11_wxGTK = wx.CURSOR_DEFAULT
    MacOS_Theme_Plus箭头 = wx.CURSOR_COPY_ARROW
    箭头及沙漏型 = wx.CURSOR_ARROWWAIT


# 边框
# https://wxpython.org/Phoenix/docs/html/wx.Window.html
class 边框:
    窗口边框 = wx.BORDER_DEFAULT  # 窗口类将决定要显示的边框类型（如果有）。

    细边框 = wx.BORDER_SIMPLE  # 在窗口周围显示细边框。wx.SIMPLE_BORDER是此样式的旧名称。

    下陷边框 = wx.BORDER_SUNKEN  # 显示下陷的边框。wx.SUNKEN_BORDER是此样式的旧名称。

    凸起边框 = wx.BORDER_RAISED  # 显示凸起的边框。wx.RAISED_BORDER是此样式的旧名称。

    静态控件边框 = wx.BORDER_STATIC  # 显示适合静态控件的边框。wx.STATIC_BORDER是此样式的旧名称。仅Windows。

    系统边框 = wx.BORDER_THEME  # 在当前平台上显示适合控件的本机边框。在Windows上，这将是一个主题边框；在大多数其他平台上，将使用凹陷的边框。有关Windows主题边框的更多信息，请参见Windows主题边框。

    无边框 = wx.BORDER_NONE  # 不显示任何边框，覆盖窗口的默认边框样式。wx.NO_BORDER是此样式的旧名称。

    # todo:: 边框样式我也不知道尽量对得上吧
    # 无边框 = wx.BORDER_NONE,
    凹入式 = wx.BORDER_SUNKEN,
    凸出式 = wx.BORDER_RAISED,
    浅凹入式 = wx.BORDER_SIMPLE,
    镜框式 = wx.BORDER_STATIC,
    单线边框式 = wx.BORDER_THEME


class 窗口边框:
    无边框 = wx.BORDER_NONE
    普通可调边框 = wx.MINIMIZE_BOX | wx.MAXIMIZE_BOX | wx.RESIZE_BORDER | wx.SYSTEM_MENU | wx.CAPTION | wx.CLOSE_BOX | wx.CLIP_CHILDREN
    普通固定边框 = wx.MINIMIZE_BOX | wx.MAXIMIZE_BOX | wx.SYSTEM_MENU | wx.CAPTION | wx.CLOSE_BOX | wx.CLIP_CHILDREN


class 底图方式:
    图片居左上 = "图片居左上"
    图片平铺 = "图片平铺"
    图片居中 = "图片居中"
    缩放图片 = "缩放图片"


class 窗口位置:
    通常 = "通常"
    居中 = "最小化"
    最小化 = "最小化"
    最大化 = "最大化"
    全屏 = "全屏"


class 播放次数:
    仅播放一次同步模式 = wx.adv.SOUND_SYNC  # Play 将阻塞并等待声音重放。
    仅播放一次 = wx.adv.SOUND_ASYNC  # 声音异步播放， Play 立即返回。
    循环播放 = wx.adv.SOUND_ASYNC | wx.adv.SOUND_LOOP  # 声音异步播放并循环播放，直到另一个声音被播放、Stop 调用或程序终止。


class 窗口样式:
    透明 = wx.TRANSPARENT_WINDOW  # 窗口是透明的，即它将不接收绘画事件。仅Windows。

    tab导航 = wx.TAB_TRAVERSAL  # wxWidgets使用此样式的窗口支持TAB其子级之间的导航，例如 wx.Dialog 和 wx.Panel。几乎不应在应用程序代码中使用它。

    支持导航键 = wx.WANTS_CHARS  # 使用它来指示窗口希望获取所有键的所有char / key事件-甚至对于通常用于对话框导航的键，例如TAB或ENTER，如果没有这种样式就不会生成这些键。如果您需要使用此样式来获取箭头等，但仍希望进行正常的键盘导航，则应调用“导航”以响应Tab和Shift-Tab的键事件。

    #  = wx.NO_FULL_REPAINT_ON_RESIZE #在Windows上，此样式用于在更改大小后完全禁用重新绘制窗口。由于现在是默认行为，因此样式现在已过时，不再起作用。

    垂直滚动条 = wx.VSCROLL  # 使用此样式可以启用垂直滚动条。请注意，此样式不能与不支持滚动条的本机控件一起使用，也不能与大多数端口中的顶级窗口一起使用。

    水平滚动条 = wx.HSCROLL  # 使用此样式启用水平滚动条。与wx.VSCROLL适用于此样式的限制相同。

    自动显示滚动条 = wx.ALWAYS_SHOW_SB  # 如果窗口具有滚动条，请在不需要滚动条时将其禁用，而不是隐藏它们（例如，当窗口的大小足够大而无需滚动条进行导航时）。目前，此样式已针对wxMSW，wxGTK和wxUniversal实现，并且在其他平台上不起作用。

    清除绘制闪烁 = wx.CLIP_CHILDREN  # 使用此样式可以消除由于重新绘制背景而在其上绘制子级而引起的闪烁。仅Windows。

    强制重绘窗口 = wx.FULL_REPAINT_ON_RESIZE  # 注意使用此样式可以在重新调整窗口大小时强制完全重绘窗口，而不是仅重绘受调整大小影响的窗口部分。请注意，默认情况下是2.5.1发行版之前的行为，并且如果您遇到以前使用过的代码重绘问题，则可能需要尝试此操作。目前，此样式仅适用于GTK + 2和Windows，并且始终在其他平台上进行完整的重新绘制。


class 按钮样式:
    文字左对齐 = wx.BU_LEFT  # 左对齐标签。仅Windows和GTK +。
    文字在顶部 = wx.BU_TOP  # 将标签对准按钮的顶部。仅Windows和GTK +。
    文字右对齐 = wx.BU_RIGHT  # 右对齐位图标签。仅Windows和GTK +。
    文字在底部 = wx.BU_BOTTOM  # 将标签对准按钮的底部。仅Windows和GTK +。
    精准贴合 = wx.BU_EXACTFIT  # 默认情况下，即使所有按钮的内容足够小以适合较小的尺寸，它们也至少由标准按钮尺寸制成。这样做是为了保持一致性，因为大多数平台在本机对话框中使用大小相同的按钮，但可以通过指定此标志来覆盖它们。如果指定了该按钮，则其大小将刚好足以容纳其内容。请注意，在MSW下，即使按钮具有非空标签，即使采用这种样式，该按钮仍将至少具有标准高度。
    不显示标签 = wx.BU_NOTEXT  # 即使按钮具有一个或其id是带有关联标签的标准库存ID之一，也无法在按钮中显示文本标签#如果不使用此样式，则仅应显示位图但使用标准ID的按钮也会显示标签。
    无边框 = wx.BORDER_NONE  # 创建一个无边框的按钮。目前已在MSW，GTK2和OSX / Cocoa中实现

    文字在左上角 = wx.BU_LEFT | wx.BU_TOP
    文字在顶部 = wx.BU_TOP
    文字在右上角 = wx.BU_RIGHT | wx.BU_TOP
    文字在左边 = wx.BU_LEFT
    文字在居中 = 0
    文字在右边 = wx.BU_RIGHT
    文字在左下角 = wx.BU_RIGHT | wx.BU_BOTTOM
    文字在底部 = wx.BU_RIGHT
    文字在右下角 = wx.BU_RIGHT | wx.BU_BOTTOM


# 文字方向
class 文本样式:
    左对齐 = wx.ALIGN_LEFT  # 将文本向左对齐。
    右对齐 = wx.ALIGN_RIGHT  # 将文本向右对齐。
    居中 = wx.ALIGN_CENTRE_HORIZONTAL  # 将文本居中（水平）。


class 标签样式:
    自动调整大小 = wx.ST_NO_AUTORESIZE  # ：默认情况下，控件将调整其大小以使其完全适合SetLabel 被调用时文本的大小。如果提供了此样式标志，则控件将不会更改其大小（此样式对于也具有ALIGN_RIGHT 或 ALIGN_CENTRE_HORIZONTAL 样式的控件特别有用， 因为否则在调用以后它们将不再有意义 SetLabel）。
    省略号在开头 = wx.ST_ELLIPSIZE_START  # ：如果labeltext宽度超过控件宽度，则用省略号替换标签的开头；用途wx.Control.Ellipsize。
    省略号在中间 = wx.ST_ELLIPSIZE_MIDDLE  # ：如果标签文本的宽度超过控件的宽度，则用省略号替换标签的中间；用途wx.Control.Ellipsize。
    省略号在末尾 = wx.ST_ELLIPSIZE_END  # ：如果标签文本的宽度超过控件的宽度，请用省略号替换标签的末尾；用途wx.Control.Ellipsize。



import base64


def base64编码(data):
    """
        支持 bytes 字节类型 和 文本类型编码为base64
    """
    if (type(data) == str):
        data_bytes = data.encode("utf-8")
    else:
        data_bytes = data
    return str(base64.b64encode(data_bytes), encoding='utf-8')

def base64解码(data):
    """
        将base64字符串 解码为 bytes 字节集
    """
    return base64.b64decode(data)




import wx
from .wxControl import *

class 滑块条(wx.Slider, 公用方法):
    pass

    def 清除刻度线(self):
        self.ClearTicks()

    def 取行大小(self):
        return self.GetLineSize()

    def 取最大位置(self):
        return self.GetMax()

    def 取最小位置(self):
        return self.GetMin()

    def 取页面间隔(self):
        return self.GetPageSize()

    def 取滑块数值范围(self):
        '返回一个元组，包含滑块的最小值跟最大值'
        return self.GetRange()

    def 取选中终点(self):
        return self.GetSelEnd()

    def 取选中起点(self):
        return self.GetSelStart()

    def 取滑块大小(self):
        return self.GetThumbLength()

    def 取刻线间隔(self):
        return self.GetTickFreq()

    def 取滑块位置(self):
        return self.GetValue()

    @组件_异常检测
    def 置滑块大小(self,数值):
        return self.SetThumbLength(数值)

    @组件_异常检测
    def 置行大小(self,数值):
        return self.SetLineSize(数值)

    @组件_异常检测
    def 置最大位置(self,数值):
        return self.SetMax(数值)

    @组件_异常检测
    def 置最小位置(self,数值):
        return self.SetMin(数值)

    @组件_异常检测
    def 置页面间隔(self,数值):
        return self.SetPageSize(数值)

    @组件_异常检测
    def 置滑块范围(self,最小值,最大值):
        return self.SetRange(最小值,最大值)

    @组件_异常检测
    def 置选中范围(self,起点值,结束值):
        return self.SetSelection(起点值,结束值)

    @组件_异常检测
    def 置刻线位置(self,位置):
        '在指定位置上标注刻线'
        return self.SetTick(位置)

    @组件_异常检测
    def 置刻线间隔(self,间隔):
        return self.SetTickFreq(间隔)

    @组件_异常检测
    def 置滑块位置(self,位置):
        return self.SetValue()



import unittest

from .imageUtil import *
from .__init__ import *
from .commonlyUtil import *
from PIL import Image


class TestImageUtil(unittest.TestCase):

    def test_1(self):
        pass
        文件路径 = 路径_合并(取运行目录(), r"example\wxefun\2.png")
        # print(文件路径)
        字节集 = 读入文件(文件路径)
        print(len(字节集))

        im = image从字节集加载(字节集)
        # print(im)
        # im.show()
        字节集 = image到字节集(im)
        print(len(字节集))


        im = image从字节集加载(字节集)
        # print(im)
        # im.show()
        字节集 = image到字节集(im)
        print(len(字节集))

        print(image取图片宽度高度(im))

        # image显示图片(im)



import wx
from .wxControl import *

class 选择列表框(wx.CheckListBox, 公用方法):
    pass
    @组件_异常检测
    def 置选中状态(self,索引,选中=True):
        self.Check(索引,选中)

    @组件_异常检测
    def 选中项目(self,索引):
        self.Check(索引,True)

    @组件_异常检测
    def 取消选中项目(self,索引):
        self.Check(索引,False)

    def 取项目数(self):
        return self.GetCount()

    def 取选中项(self):
        '返回一个元组,包含所有选中的表项索引'
        return self.GetCheckedItems()

    def 取选中项_文本(self):
        '返回一个元组,包含所有选中的表项文本'
        return self.GetCheckedStrings()

    @组件_异常检测
    def 是否选中(self,索引):
        return self.IsChecked(索引)

    @组件_异常检测
    def 置选中状态_批量(self,索引列表):
        '传入需要选中的索引列表,不存在的会报错'
        self.SetCheckedItems(索引列表)

    @组件_异常检测
    def 置选中状态_文本_批量(self,索引列表):
        '传入需要选中的项目文本列表,不存在的会报错'
        self.SetCheckedStrings(索引列表)

    @组件_异常检测
    def 置指定项目背景色(self, 索引, 颜色):
        '在列表框中设置项目的背景色。仅在MSW上且wx.LB_OWNERDRAW设置了标志时有效。'
        self.SetItemBackgroundColour(索引, 颜色)
        self.Refresh()

    @组件_异常检测
    def 置指定项目前景色(self, 索引, 颜色):
        '在列表框中设置项目的前景色。仅在MSW上且wx.LB_OWNERDRAW设置了标志时有效。'
        self.SetItemForegroundColour(索引, 颜色)
        self.Refresh()

    @组件_异常检测
    def 置指定项目字体(self, 索引, 字体):
        '在列表框中设置项目的字体。仅在MSW上且wx.LB_OWNERDRAW设置了标志时有效。'
        self.SetItemFont(索引, 字体)
        self.Refresh()

    @组件_异常检测
    def 置顶指定项(self,索引):
        '将指定的项目设置为第一个可见项目。'
        self.SetFirstItem(索引)

    def 取首个可见项索引(self):
        '返回最顶部可见项目的索引。'
        return self.GetTopItem()

    @组件_异常检测
    def 取指定坐标索引(self,左边,顶边):
        '返回列表框内指定坐标处项目索引'
        return self.HitTest(左边,顶边)

    @组件_异常检测
    def 插入项目(self,插入位置,项目列表):
        return self.Insert(项目列表,插入位置)

    @组件_异常检测
    def 插入项目2(self,插入位置,项目列表):
        return self.InsertItems(项目列表,插入位置)

    def 清空表项(self):
        self.Clear()

    @组件_异常检测
    def 置项目列表(self,项目列表):
        '会覆盖原有的项目列表'
        self.SetItems(项目列表)

    @组件_异常检测
    def 加入项目(self,项目):
        '支持单个或多个项目,多个项目使用列表传入，加入后会返回最后一个项目索引'
        return self.Append(项目)

    @组件_异常检测
    def 加入项目2(self,项目):
        '支持单个或多个项目,多个项目使用列表传入'
        self.AppendItems(项目)

    @组件_异常检测
    def 删除指定项目(self,索引):
        self.Delete(索引)



# -*- coding:utf-8 -*-
import pymysql,re
from .public import *

#事务回滚等操作需要支持库使用INNODB引擎

@异常处理返回类型逻辑型
def 连接Mysql(连接地址, 用户名, 密码, 数据库名, 端口号,自动提交=True, 编码='utf8'):
    '成功返回连接对象,开启 自动提交 后不能使用保留点操作'
    db = pymysql.connect(host=连接地址, port=int(端口号), user=用户名, passwd=密码, db=数据库名, charset=编码,autocommit=自动提交)
    if db:
        return 数据库类(db)
    return False



class 游标类:
    def __init__(self,游标对象):
        '传入连接Mysql后通过cursor获取的游标'
        self.__cursor = 游标对象

    @异常处理返回类型逻辑型
    def 关闭游标(self):
        self.__cursor.close()
        return True

    @异常处理返回类型逻辑型
    def 取全部记录(self):
        '一次读取全部数据，如果管道内没有数据，则返回空元组或空列表'
        return self.__cursor.fetchall()

    @异常处理返回类型逻辑型
    def 取下一条记录(self):
        '一次一条数据'
        return self.__cursor.fetchone()

    @异常处理返回类型逻辑型
    def 取多条记录(self,数量=1):
        '一次多条数据，括号内填入要读取的数据条数。不填则为1条数据，如果读数超过实际条数，只显示实际条数'
        return self.__cursor.fetchmany(数量)

    @异常处理返回类型逻辑型
    def 移到指定位置(self, 位置):
        '首位为0。（第0位为第一条数据）'
        return self.__cursor.scroll(位置,mode='absolute')

    @异常处理返回类型逻辑型
    def 移到相对位置(self, 位置):
        '例：传入3则表示移到当前位置往后第三条记录的位置上'
        return self.__cursor.scroll(位置, mode='relative')

    @异常处理返回类型逻辑型
    def 取当前指针位置(self):
        '上次光标位置'
        return self.__cursor.lastrowid

    @异常处理返回类型逻辑型
    def 执行SQL语句(self,sql):
        '返回受影响的行数'
        return self.__cursor.execute(sql)

    @异常处理返回类型逻辑型
    def 增加记录_批量(self,表名,字段名列表,数据列表):
        '''
        用于增加多条记录等操作，效率比execute快很多
        :param 表名: user
        :param 字段名列表: ['user','pwd']
        :param 数据列表: [('123456','a11111'),('56789','aaaaa1')]
        :return: 返回受影响的行数
        '''
        字段 = "`{}`".format("`,`".join(字段名列表))
        sql = "insert into `{}` ({}) value ({})".format(表名,字段,('%s,'*len(字段名列表))[:-1])
        return self.__cursor.executemany(sql,数据列表)

    @异常处理返回类型逻辑型
    def 增加记录(self, 表名, 字段名列表, 数据列表):
        '''
        :param 表名: user
        :param 字段名列表: ['user','pwd']
        :param 数据列表: [('123456','a11111'),('56789','aaaaa1')]
        :return: 返回受影响的行数
        '''
        字段 = "`{}`".format("`,`".join(字段名列表))
        sql = "insert into `{}` ({}) value {}".format(表名, 字段, repr(数据列表)[1:-1])
        return self.__cursor.execute(sql)

    @异常处理返回类型逻辑型
    def 追加内容(self, 表名, 字段名, 加入的内容,加入位置=0,条件=''):
        '加入位置:0.前面 1.后面   条件可空，空为全部操作'
        条件 += ' where ' if 条件 else ''
        if 加入位置:
            sql = "update {} set {} = concat({},{}){};".format(表名,字段名,字段名,加入的内容,条件)
        else:
            sql = "update {} set {} = concat({},{}){};".format(表名,字段名,加入的内容,字段名,条件)
        return self.__cursor.execute(sql)

    @异常处理返回类型逻辑型
    def 查找记录_按时间范围(self, 表名,字段名,开始时间,结束时间,时间格式='%Y-%m-%d %H:%M:%S'):
        sql = "select * from {} where from_unixtime({},'{}') between '{}' and '{}';".format(表名,字段名,时间格式,开始时间,结束时间)
        return self.__cursor.execute(sql)

    @异常处理返回类型逻辑型
    def 查找记录_指定日期(self, 表名,字段名,日期,时间格式='%Y-%m-%d %H:%M:%S'):
        sql = "select * from {} where from_unixtime({},'{}') = '{}'".format(表名,字段名,时间格式,日期)
        return self.__cursor.execute(sql)

    @异常处理返回类型逻辑型
    def 查找记录_取记录数(self, 表名, 查找条件='', 模糊查找=False):
        '''
        :param 表名: 表名
        :param 查找条件: id='123' and user='456'
        :param 模糊查找:
        :return: 返回记录条数
        '''
        if 模糊查找:
            z = re.findall("='.*?'", 查找条件)
            for x in z:
                查找条件 = 查找条件.replace(x, x[:-1].replace("='", " like '%") + "%'")
            sql = "select count(*) from {} where {}".format(表名, 查找条件)
        else:
            if 查找条件:
                sql = "select count(*) from {} where {}".format(表名, 查找条件)
            else:
                sql = "select count(*) from {}".format(表名)
        self.__cursor.execute(sql)
        return self.__cursor.fetchone()[0]

    @异常处理返回类型逻辑型
    def 查找记录(self, 表名, 字段名, 查找条件='',模糊查找=False, 取出的数量=0, 排序=""):
        '''
        :param 表名: 表名
        :param 字段名: id,user  需要返回的字段信息 * 为全部
        :param 查找条件: id='123' and user='456'
        :param 模糊查找:
        :param 取出的数量:
        :param 排序: id asc(根据ID字段正序排列)，id desc(根据ID字段倒序排列)
        :return: 返回受影响的行数
        '''
        if 模糊查找:
            z = re.findall("='.*?'", 查找条件)
            for x in z:
                查找条件 = 查找条件.replace(x, x[:-1].replace("='", " like '%") + "%'")
            sql = "select {} from {} where {}".format(字段名, 表名, 查找条件)
        else:
            if 查找条件:
                sql = "select {} from {} where {}".format(字段名,表名,查找条件)
            else:
                sql = "select {} from {}".format(字段名,表名)

        if 排序:
            sql += " order by " + 排序

        if 取出的数量:
            sql += " limit " + str(取出的数量)
        return self.__cursor.execute(sql)

    @异常处理返回类型逻辑型
    def 更新记录(self, 表名, 更新条件, 更新内容,模糊匹配=False):
        if 模糊匹配:
            z = re.findall("='.*?'", 更新条件)
            for x in z:
                更新条件 = 更新条件.replace(x, x[:-1].replace("='", " like '%") + "%'")
            sql = "update {} set {} where {}".format(表名,更新内容,更新条件)
        else:
            sql = "update {} set {} where {}".format(表名,更新内容,更新条件)
        return self.__cursor.execute(sql)

    @异常处理返回类型逻辑型
    def 删除记录(self, 表名, 删除条件, 模糊匹配=False):
        if 模糊匹配:
            z = re.findall("='.*?'", 删除条件)
            for x in z:
                删除条件 = 删除条件.replace(x,x[:-1].replace("='", " like '%")+"%'")
            sql = "delete from {} where {}".format(表名, 删除条件)
        else:
            sql = "delete from {} where {}".format(表名, 删除条件)
        return self.__cursor.execute(sql)

    @异常处理返回类型逻辑型
    def 取全部表名(self, 库名):
        '返回格式[表名,表名]'
        sql = "show tables from " + 库名
        self.__cursor.execute(sql)
        return [x[0] for x in self.__cursor.fetchall()]

    @异常处理返回类型逻辑型
    def 取全部字段名(self,表名,只取字段名=True):
        '默认返回格式[字段名,字段名...]'
        sql = "select * from " + 表名
        self.__cursor.execute(sql)
        if 只取字段名:
            return [x[0] for x in self.__cursor.description]
        else:
            return self.__cursor.description

    @异常处理返回类型逻辑型
    def 事务_开启(self):
        sql = "start transaction"
        self.__cursor.execute(sql)
        return True

    @异常处理返回类型逻辑型
    def 事务_设置保留点(self,保留点名称):
        sql = "savepoint " + 保留点名称
        self.__cursor.execute(sql)
        return True

    @异常处理返回类型逻辑型
    def 事务_保留点回滚(self, 保留点名称):
        sql = "rollback to " + 保留点名称
        self.__cursor.execute(sql)
        return True



class 数据库类:
    def __init__(self,连接对象):
        '传入连接Mysql后返回的对象'
        self.__dc = 连接对象

    @异常处理返回类型逻辑型
    def 获取游标(self,返回字典类型=False):
        '将取回值以字典形式显示,默认为False'
        if 返回字典类型:
            return 游标类(self.__dc.cursor(pymysql.cursors.DictCursor))
        else:
            return 游标类(self.__dc.cursor())


    @异常处理返回类型逻辑型
    def 断开连接(self):
        self.__dc.close()
        return True

    @异常处理返回类型逻辑型
    def 事务_提交(self):
        '执行成功返回True'
        self.__dc.commit()
        return True

    @异常处理返回类型逻辑型
    def 事务_回滚(self):
        '执行成功返回True'
        self.__dc.rollback()
        return True


    def 是否在线(self):
        '在线返回True'
        try:
            self.__dc.ping(reconnect=True)
            return True
        except:
            return False




"""

.. Hint::
    gevent实现的协程池

    调用时需要引入猴子补丁 需要安装gevent

    pip install gevent

    代码中需要引入猴子补丁

    import gevent
    from gevent import monkey
    monkey.patch_all()

    还有另一个版本的协程池实现 asyncPool 为python原生支持的实现

.. literalinclude:: ../../../pyefun/asyncPool/asyncPool_test.py
    :language: python
    :caption: 代码示例
    :linenos:

"""
import gevent
from gevent import monkey
from gevent.threadpool import ThreadPool
monkey.patch_all()


class 协程池():
    """
        创建一个协程池运行任务

        pool = 协程池(协程数量=2)
        任务列表 = []
        for i in range(10):
            任务列表.append(pool.创建任务(工作线程4,i))

        pool.运行任务(任务列表)

    """

    def __init__(self, 协程数量):
        self.pool = ThreadPool(协程数量)

    def 等待协程完成任务(self, 任务列表):
        gevent.joinall(任务列表)

    def 投递任务(self, 任务函数, *args, **kwargs):
        return self.pool.spawn(任务函数, *args, **kwargs)

    def 等待(self):
        """等待所有任务完成"""
        self.pool.join()
        # gevent.wait()

    def 关闭(self):
        """等待所有任务完成"""
        self.pool.kill()



import wx.adv
import wx
from .wxControl import *

class 日期框(wx.adv.DatePickerCtrl, 公用方法):
    pass

    def 取可选日期范围(self):
        return self.GetRange()

    @组件_异常检测
    def 置可选日期范围(self,最早日期,最晚日期):
        return self.SetRange(最早日期,最晚日期)

    def 取当前日期(self):
        return self.GetValue()

    @组件_异常检测
    def 置当前日期(self,日期):
        return self.SetValue(日期)



import wx
from .wxControl import *


class 单选框(wx.RadioButton, 公用方法):
    pass

    def 是否选中(self):
        return self.GetValue()

    @组件_异常检测
    def 置选中状态(self, 状态):
        'True/False'
        return self.SetValue(状态)



import wx
from .wxControl import *


class 复选框(wx.CheckBox, 公用方法):
    pass

    def 取三态多选框状态(self):
        '返回 0.未选中  1.选中  2.半选中'
        return self.Get3StateValue()

    @组件_异常检测
    def 置三态多选框状态(self,状态):
        '0.未选中  1.选中  2.半选中'
        return self.Set3StateValue(状态)

    def 是否选中(self):
        return self.GetValue()

    @组件_异常检测
    def 置选中状态(self,状态):
        'True/False'
        return self.SetValue(状态)

    def 是否为三态复选框(self):
        return self.Is3State()

    def 是否可设置为半选中(self):
        return self.Is3rdStateAllowedForUser()


class 选择框(复选框):
    pass





import wx.lib.buttons as lib_button
from .wxControl import *


class 图文按钮(lib_button.ThemedGenBitmapTextButton, 公用方法):
    pass

    def 取禁用状态图片(self):
        return self.GetBitmapDisabled()

    def 取焦点状态图片(self):
        return self.GetBitmapFocus()

    def 取正常状态图片(self):
        return self.GetBitmapLabel()

    def 取按下状态图片(self):
        return self.GetBitmapSelected()

    @组件_异常检测
    def 置禁用状态图片(self,图片):
        return self.SetBitmapDisabled(图片)

    @组件_异常检测
    def 置焦点状态图片(self,图片):
        return self.SetBitmapFocus(图片)

    @组件_异常检测
    def 置正常状态图片(self,图片):
        return self.SetBitmapLabel(图片)

    @组件_异常检测
    def 置按下状态图片(self,图片):
        return self.SetBitmapSelected(图片)



import wx
from .wxControl import *


class 容器(wx.Panel, 公用方法):
    底图方式 = "图片居左上"

    @property
    def 底图(self):
        return self.底图文件路径

    @底图.setter
    def 底图(self, value):
        if value == "":
            return
        self.底图文件路径 = value
        self.Bind(wx.EVT_ERASE_BACKGROUND, self._绘制底图)
        self.Bind(wx.EVT_SIZE, self._onSize)

    def _onSize(self, evt):
        self.Refresh()

    def _绘制底图(self, event):
        dc = event.GetDC()
        if not dc:
            dc = wx.ClientDC(self)
            rect = self.GetUpdateRegion().GetBox()
            dc.SetClippingRect(rect)
        dc.Clear()
        size = self.GetClientSize()

        bmp = wx.Image(self.底图文件路径, wx.BITMAP_TYPE_ANY)
        if self.底图方式 == "图片居左上":
            pass
        elif self.底图方式 == "缩放图片":
            bmp = bmp.Scale(size.x, size.y)
        elif self.底图方式 == "图片平铺":
            pass
            # todo:: 未完成的代码 图片平铺
        elif self.底图方式 == "图片居中":
            pass
            # todo:: 未完成的代码 图片居中

        dc.DrawBitmap(bmp.ConvertToBitmap(), 0, 0)

    _Esc键关闭 = False

    @property
    def Esc键关闭(self):
        return self._Esc键关闭

    @Esc键关闭.setter
    def Esc键关闭(self, value):
        self._Esc键关闭 = value
        if value:
            self.Bind(wx.EVT_KEY_DOWN, self._OnKeyDown)

    def _OnKeyDown(self, e):
        key = e.GetKeyCode()
        if key == wx.WXK_ESCAPE:
            exit()


class 面板(容器):
    pass




import wx.lib.agw.hyperlink as hyperlink

import wx.adv
import wx
from .wxControl import *

class 超级链接框(wx.adv.HyperlinkCtrl, 公用方法):
    pass

    def 取单击前焦点颜色(self):
        '返回鼠标悬停在控件上时用于打印超链接标签的颜色。'
        return self.GetHoverColour()

    def 取初始颜色(self):
        '返回以前从未单击过链接（即尚未访问链接）并且鼠标不在控件上时用于打印标签的颜色。'
        return self.GetNormalColour()

    def 取URL(self):
        '返回与超链接关联的URL。'
        return self.GetURL()

    def 是否已点击(self):
        '返回True超链接是否已被用户至少单击一次。'
        return self.GetVisited()

    def 取单击后焦点颜色(self):
        '返回鼠标悬停在控件上且之前已单击链接（即已访问链接）时用于打印标签的颜色。'
        return self.GetVisitedColour()

    @组件_异常检测
    def 置焦点颜色(self,颜色):
        '设置鼠标悬停在控件上时用于打印超链接标签的颜色。'
        return self.SetHoverColour(颜色)

    @组件_异常检测
    def 置初始颜色(self,颜色):
        '设置以前从未单击过链接（即未访问链接）并且鼠标不在控件上时用于打印标签的颜色。'
        return self.SetNormalColour(颜色)

    @组件_异常检测
    def 置URL(self,url):
        '设置与超链接关联的URL。'
        return self.SetURL(url)

    @组件_异常检测
    def 置访问状态(self,已访问=True):
        '将超链接标记为已访问/未访问'
        return self.SetVisited(已访问)

    @组件_异常检测
    def 置已点击颜色(self,颜色):
        return self.SetVisitedColour(颜色)

class 超级链接框L(hyperlink.HyperLinkCtrl, 公用方法):
    pass
    @组件_异常检测
    def 允许打开链接(self,打开=True):
        '单击后自动浏览到URL。'
        return self.AutoBrowse(True)

    @组件_异常检测
    def 弹出错误提示(self,提示内容):
        return self.DisplayError(提示内容)

    @组件_异常检测
    def 允许右键弹出菜单(self,弹出=True):
        return self.DoPopup(弹出)

    @组件_异常检测
    def 允许翻转(self,允许=False):
        '干啥用的'
        return self.EnableRollover(允许)

    def 标题是否为粗体(self):
        return self.GetBold()

    def 取默认字体颜色(self):
        return self.GetColours()[0]

    def 取访问后字体颜色(self):
        return self.GetColours()[1]

    def 取焦点字体颜色(self):
        return self.GetColours()[2]

    def 取各种字体颜色(self):
        '返回一个元组,(默认颜色,点击后颜色,焦点颜色)'
        return self.GetColours()

    def 默认标题是否带下划线(self):
        return self.GetUnderlines()[0]

    def 焦点标题是否带下划线(self):
        return self.GetUnderlines()[1]

    def 点击后标题是否带下划线(self):
        return self.GetUnderlines()[2]

    def 标题是否带各状态下划线(self):
        '返回一个元组，显示各状态下标题是否带下划线,（默认标题，焦点状态标题，点击后标题）'
        return self.GetUnderlines()

    def 取鼠标光标(self):
        return self.GetLinkCursor()

    def 取URL(self):
        return self.GetURL()

    def 是否已访问过(self):
        return self.GetVisited()

    @组件_异常检测
    def 打开指定链接(self,链接):
        return self.GotoURL(链接)

    @组件_异常检测
    def 置标题字体粗细(self,粗体=True):
        return self.SetBold(粗体)

    @组件_异常检测
    def 置各状态标题颜色(self,默认,访问后,焦点):
        '设置链接，访问的链接和鼠标悬停的颜色。'
        return self.SetColours(默认,访问后,焦点)

    @组件_异常检测
    def 置默认标题颜色(self,颜色):
        return self.SetColours(颜色,None,None)

    @组件_异常检测
    def 置访问后标题颜色(self,颜色):
        return self.SetColours(None,颜色,None)

    @组件_异常检测
    def 置焦点标题颜色(self, 颜色):
        return self.SetColours(None, None, 颜色)

    @组件_异常检测
    def 置各状态标题下划线(self,默认=False,已访问=False,焦点=False):
        '设置是否应为新链接，访问的链接和过渡文本加下划线。'
        return self.SetUnderlines(默认,已访问,焦点)

    @组件_异常检测
    def 置默认标题下划线(self,下划线=True):
        return self.SetUnderlines(下划线,None,None)

    @组件_异常检测
    def 置访问猴标题下划线(self,下划线=True):
        return self.SetUnderlines(None,下划线,None)

    @组件_异常检测
    def 置焦点标题下划线(self,下划线=True):
        return self.SetUnderlines(None,None,下划线)

    @组件_异常检测
    def 置URL(self,url):
        return self.SetURL(url)

    @组件_异常检测
    def 置访问状态(self,状态=True):
        '设置链接访问状态是否已访问郭'
        return self.SetVisited(状态)

    @组件_异常检测
    def 更新链接(self,刷新控件=True):
        self.UpdateLink(刷新控件)



"""
剪切板操作

pip install pyperclip

"""
import pyperclip

def 取剪辑板文本() -> str:
    return pyperclip.paste()


def 置剪辑板文本(str):
    pyperclip.copy(str)


def 剪辑板中可有文本() -> bool:
    return 取剪辑板文本() != ""


def 清除剪辑板():
    pyperclip.copy("")


from pyefun.asyncPoolGevent import *
import time
from pyefun.__init__ import *

def 工作线程4(参数):
    with 时间统计() as t:
        延时(取随机数(4, 4))
        # 延时(取随机数(5, 5))
        print(time.time(),t.取耗时(), 参数, "线程类测试")
        # print(取现行时间(), t.取耗时(), 参数, "线程类测试")

pool = 协程池(协程数量=9)
for i in range(11):
    print("创建任务", i)
    pool.投递任务(工作线程4, i)
pool.等待()


import urllib.parse


def url编码(欲编码的文本, encoding="utf-8"):
    """
    encoding 可以设置为 gbk
    """
    return urllib.parse.quote(欲编码的文本, encoding=encoding)


def url解码(url,encoding="utf-8"):
    """
    encoding 可以设置为 gbk
    """
    return urllib.parse.unquote(url, encoding=encoding)



import unittest

from .url import *


class TestUrl(unittest.TestCase):

    def test_1(self):
        data = url编码("你好123456798", "gbk")
        self.assertEqual(data, "%C4%E3%BA%C3123456798")
        data = url解码(data, "gbk")
        self.assertEqual(data, "你好123456798")

        data = url编码("你好123456798")
        self.assertEqual(data, "%E4%BD%A0%E5%A5%BD123456798")
        data = url解码(data)
        self.assertEqual(data, "你好123456798")

        data = url编码("你好123456798")
        self.assertEqual(data, "%E4%BD%A0%E5%A5%BD123456798")
        data = url解码(data)
        self.assertEqual(data, "你好123456798")



import ubelt.timerit
from collections import OrderedDict

class 计时统计(ubelt.timerit.Timerit):
    """
    计代码块的平均时间
    """

    def __init__(self, 次数=1, 标签=None, 取样=3, 单位=None, 显示信息=None):
        self.t = super().__init__(次数, 标签, 取样, 单位, 显示信息)

    def 取耗时(self):
        return self.total_time

    def report(self, verbose=1):
        """
        Creates a human readable report

        Args:
            verbose (int): verbosity level. Either 1, 2, or 3.

        Returns:
            str: the report

        SeeAlso:
            :func:`Timerit.print`

        Example:
            >>> import math
            >>> ti = Timerit(num=1).call(math.factorial, 5)
            >>> print(ti.report(verbose=1))
            Timed best=...s, mean=...s
        """
        lines = []
        if verbose >= 2:
            # use a multi-line format for high verbosity
            lines.append(self._status_line(tense='past'))
            if verbose >= 3:
                unit, mag = _choose_unit(self.total_time, self.unit,
                                         self._asciimode)
                lines.append('    总耗时: {total:.{pr}{t}} {unit}'.format(
                    total=self.total_time / mag,
                    t=self._precision_type,
                    pr=self._precision, unit=unit))
            lines.append('    统计每次时间: {}'.format(self._seconds_str()))
        else:
            # use a single-line format for low verbosity
            line = 'Timed ' + self._seconds_str()
            if self.label:
                line += ' for ' + self.label
            lines.append(line)
        text = '\n'.join(lines)
        return text

    def _seconds_str(self):
        """
        Returns:
            str: human readable text

        Example:
            >>> self = Timerit(num=100, bestof=10, verbose=0)
            >>> self.call(lambda : sum(range(100)))
            >>> print(self._seconds_str())
            ... 'best=3.423 µs, ave=3.451 ± 0.027 µs'
        """
        mean = self.mean()
        unit, mag = _choose_unit(mean, self.unit, self._asciimode)

        unit_min = self.min() / mag
        unit_mean = mean / mag

        # Is showing the std useful? It probably doesn't hurt.
        std = self.std()
        unit_std = std / mag
        pm = _trychar('±', '+-', self._asciimode)
        fmtstr = ('最佳={min:.{pr1}{t}} {unit}, '
                  '平均={mean:.{pr1}{t}} {pm} {std:.{pr2}{t}} {unit}')
        pr1 = pr2 = self._precision
        if isinstance(self._precision, int):  # pragma: nobranch
            pr2 = max(self._precision - 2, 1)
        unit_str = fmtstr.format(min=unit_min, unit=unit, mean=unit_mean,
                                 t=self._precision_type, pm=pm, std=unit_std,
                                 pr1=pr1, pr2=pr2)
        return unit_str

    def _status_line(self, tense='past'):
        """
        Text indicating what has been / is being done.

        Example:
            >>> print(Timerit()._status_line(tense='past'))
            Timed for: 1 loops, best of 1
            >>> print(Timerit()._status_line(tense='present'))
            Timing for: 1 loops, best of 1
        """
        action = {'past': '计时统计', 'present': '计时开始'}[tense]
        line = '{action} {label}次数: {num:d} 循环, 取样 {bestof:d}'.format(
            label=self.label + ' ' if self.label else '',
            action=action, num=self.num, bestof=min(self.bestof, self.num))
        return line



def _trychar(char, fallback, asciimode=None):
    """
    Logic from IPython timeit to handle terminals that cant show mu

    Args:
        char (str): character, typically unicode, to try to use
        fallback (str): ascii character to use if stdout cannot encode char
        asciimode (bool): if True, always use fallback

    Example:
        >>> char = _trychar('µs', 'us')
        >>> print('char = {}'.format(char))
        >>> assert _trychar('µs', 'us', asciimode=True) == 'us'

    """
    if asciimode is True:
        # If we request ascii mode simply return it
        return fallback
    if hasattr(sys.stdout, 'encoding') and sys.stdout.encoding:  # pragma: nobranch
        try:
            char.encode(sys.stdout.encoding)
        except Exception:  # nocover
            pass
        else:
            return char
    return fallback  # nocover


def _choose_unit(value, unit=None, asciimode=None):
    """
    Finds a good unit to print seconds in

    Args:
        value (float): measured value in seconds
        unit (str): if specified, overrides heuristic decision
        asciimode (bool): if True, forces ascii for microseconds

    Returns:
        tuple[(str, float)]: suffix, mag:
            string suffix and conversion factor

    Example:
        >>> assert _choose_unit(1.1, unit=None)[0] == 's'
        >>> assert _choose_unit(1e-2, unit=None)[0] == 'ms'
        >>> assert _choose_unit(1e-4, unit=None, asciimode=True)[0] == 'us'
        >>> assert _choose_unit(1.1, unit='ns')[0] == 'ns'
    """
    micro = _trychar('µs', 'us', asciimode)
    units = OrderedDict([
        ('s', ('s', 1e0)),
        ('ms', ('ms', 1e-3)),
        ('us', (micro, 1e-6)),
        ('ns', ('ns', 1e-9)),
    ])
    if unit is None:
        for suffix, mag in units.values():  # pragma: nobranch
            if value > mag:
                break
    else:
        suffix, mag = units[unit]
    return suffix, mag



"""

.. Hint::
    JavaScript引擎

    在python中运行JavaScript代码

    python调用js的函数

    需要安装 pip install PyExecJS

    需要安装 node 环境

    - PyV8 = "PyV8"
    - Node = "Node"
    - JavaScriptCore = "JavaScriptCore"
    - SpiderMonkey = "SpiderMonkey"
    - JScript = "JScript"
    - PhantomJS = "PhantomJS"
    - SlimerJS = "SlimerJS"
    - Nashorn = "Nashorn"

.. literalinclude:: ../../../pyefun/javscript/javscript_test.py
    :language: python
    :caption: 代码示例
    :linenos:

"""

import execjs


class javscript:
    """
    python调用js的函数
    需要安装 pip install PyExecJS
    需要安装 node 环境
        PyV8 = "PyV8"
        Node = "Node"
        JavaScriptCore = "JavaScriptCore"
        SpiderMonkey = "SpiderMonkey"
        JScript = "JScript"
        PhantomJS = "PhantomJS"
        SlimerJS = "SlimerJS"
        Nashorn = "Nashorn"
    """

    def __init__(self, 运行环境名称: str = "Node"):
        self.设置运行环境(运行环境名称)

    def 取运行环境(self):
        return self.jscript.name

    def 设置运行环境(self, 运行环境名称: str = "Node"):
        self.jscript = execjs.get(运行环境名称)

    def 加载代码(self, js代码: str):
        self.jscript = self.jscript.compile(js代码)

    def 执行(self, 函数名: str, *参数):
        return self.jscript.call(函数名, *参数)

    def 运行(self, js代码: str):
        return self.jscript.eval(js代码)


def 运行js(加载js的代码, 运行js代码, 运行环境名称: str = "Node"):
    ctx = execjs.get(运行环境名称).compile(加载js的代码)
    return ctx.eval(运行js代码)



import unittest

from .arithmeticOperationBase import *
from .typeConv import *

class TestDirk(unittest.TestCase):

    def test_1(self):
        pass
        self.assertEqual(四舍五入(123.144, 2), 123.14)
        self.assertEqual(四舍五入(123.146, 2), 123.15)
        self.assertEqual(取绝对值(-1), 1)

        self.assertEqual(取绝对值(-100), 100)
        self.assertEqual(取绝对值(-0.1), 0.1)
        self.assertEqual(取绝对值(0.1), 0.1)

        self.assertEqual(取整(100.111), 100)
        self.assertEqual(到整数('8852791.5'), 8852791)
        self.assertEqual(到整数('-8852791.5'), -8852791)
        self.assertEqual(到整数('8852791.5123456789'), 8852791)
        self.assertEqual(到整数('-8852791.5123456789'), -8852791)

        self.assertEqual(求次方(2, 2), 4)
        self.assertEqual(求次方(4, 2), 16)
        self.assertEqual(求余弦(30), 0.15425144988758405)
        self.assertEqual(求反正切(30), 1.5374753309166493)
        self.assertEqual(求平方根(2), 1.4142135623730951)
        self.assertEqual(求正切(30), -6.405331196646276)
        self.assertEqual(求正弦(30), -0.9880316240928618)
        self.assertEqual(取随机数(1, 1), 1)
        self.assertEqual(取整数('8852791.5'), 8852791)
        self.assertEqual(取整数('+8852791.5'), 8852791)
        self.assertEqual(取整数('-8852791.5'), -8852791)
        self.assertEqual(取整数(8852791.5), 8852791)




"""
阿里云 oss操作

pip install oss2

https://github.com/aliyun/aliyun-oss-python-sdk?spm=a2c4g.11186623.2.4.391446a1tSs54t

"""
import oss2
from itertools import islice


class 获取授权(oss2.Auth):
    """签名版本1
    """

    def __init__(self, access_key_id, access_key_secret):
        super(获取授权, self).__init__(access_key_id, access_key_secret)


class 初始化Bucket(oss2.Bucket):
    def __init__(self, auth, endpoint, bucket_name,
                 is_cname=False,
                 session=None,
                 connect_timeout=None,
                 app_name='',
                 enable_crc=True):
        super(初始化Bucket, self).__init__(auth, endpoint, bucket_name,
                                     is_cname,
                                     session,
                                     connect_timeout,
                                     app_name,
                                     enable_crc)

    def 上传文件(self, key, data,
             headers=None,
             progress_callback=None):
        return self.put_object(key, data,
                               headers,
                               progress_callback)

    def 获取文件(self, key,
             byte_range=None,
             headers=None,
             progress_callback=None,
             process=None,
             params=None):
        return self.get_object(key,
                               byte_range,
                               headers,
                               progress_callback,
                               process,
                               params)

    def 删除文件(self, key, params=None, headers=None):
        return self.delete_object(key, params=None, headers=None)

    def 上传文件从文件路径(self, key, filename,
                  headers=None,
                  progress_callback=None):
        return self.put_object_from_file(key, filename,
                                         headers,
                                         progress_callback)

    def 下载文件(self, key, filename,
             byte_range=None,
             headers=None,
             progress_callback=None,
             process=None,
             params=None):
        return self.get_object_to_file(key, filename,
                                       byte_range,
                                       headers,
                                       progress_callback,
                                       process,
                                       params)

    def 列举文件(self, 前缀='', delimiter='', marker='', max_keys=100, max_retries=None, headers=None):
        # oss2.ObjectIterator用于遍历文件。
        list = []
        for b in islice(oss2.ObjectIterator(self,
                                            prefix=前缀,
                                            delimiter=delimiter,
                                            marker=marker,
                                            max_keys=max_keys,
                                            max_retries=max_retries,
                                            headers=headers
                                            ), 10):
            # print(b.key)
            list.append(b.key)
        return list



import binascii

def binary编码(data):
    if (type(data) == str):
        data_bytes = data.encode("utf-8")
    else:
        data_bytes = data
    return binascii.b2a_hex(data_bytes)


def binary解码(data):
    return binascii.a2b_hex(data)
